package minicollection;

import java.util.*;


public class AbstractImplJCBagPairsAdaptor<K,V> 
		implements IMiniBagPairsImplementation<K, V> {

	private Map<K, IMiniBagSingles<V>> adaptee = null;
	
	public AbstractImplJCBagPairsAdaptor(Map<K, IMiniBagSingles<V>> adaptee) {
		this.adaptee = adaptee;
	}
	
	public int implSize() {
		// keys * counts
		
		List<IMiniBagSingles<V>> valuePairs = 
			new ArrayList<IMiniBagSingles<V>>(adaptee.values());
		Integer count = 0;
		
		for (IMiniBagSingles<V>  pairs : valuePairs) {
			count += pairs.size();
		}
 		return count;
	}
	
	public boolean implContains(Object key) {
		return adaptee.containsKey(key);
	}

	public int implCount(Object key) {
		int count = 0;
		IMiniBagSingles<V> valuePairs = adaptee.get(key);
		if (valuePairs != null) {
			count = valuePairs.size();
		}
		return count;
	}

	public boolean implIsEmpty() {
		return adaptee.isEmpty();
	}
	
	// one copy of each mapping
	public void implRemoveDuplicates() {
		Set<Map.Entry<K, IMiniBagSingles<V>>> pairs = adaptee.entrySet();
			for (Map.Entry<K, IMiniBagSingles<V>> pair : pairs) {
				pair.getValue().removeDuplicates();
			}
			
	}	// this would mean setting all the keys to 1!
	
	public boolean implClear() {
		adaptee.clear();
		return adaptee.isEmpty();
	}
	
	public Iterator<IMiniKeyValuePair<IMiniKeyValuePair<K,V>,Integer>> 
					implIterator() {
		return new ImplJCIteratorForBagPairsAdaptor<K,V>(implBagIterator());
	}
	
	public IMiniIterator<IMiniKeyValuePair<IMiniKeyValuePair<K,V>,Integer>> 
					implMiniIterator() {
		return implBagIterator();
	}

	public int implUniqueKeySize() {
		return adaptee.size();
	}

	public int implUniqueKeyValuePairSize() {
		List<IMiniBagSingles<V>> valuePairs = 
			new ArrayList<IMiniBagSingles<V>>(adaptee.values());
		Integer count = 0;
		
		for (IMiniBagSingles<V>  pairs : valuePairs) {
			count += pairs.uniqueKeySize();
		}
			return count;
	}
	
    public InsertCheck implAdd(K key, V value) {
    	InsertCheck isAdded = InsertCheck.FAILED;
    	
    	IMiniBagSingles<V> values = adaptee.get(key);
    	if (values != null) {
    		isAdded = values.add(value);
    	}
    	else {
    		IFactoryMiniBagSingles<V> factory = new FactoryMiniBagSingles<V>();
    		values = factory.makeBagSingles();
    		InsertCheck addedValue = values.add(value);
    		if (addedValue != InsertCheck.FAILED) {
    			adaptee.put(key, values);
    			isAdded = implContainsPair(key, value) ? InsertCheck.NEW : InsertCheck.FAILED;
    		}
    	}
    	return isAdded;
    }


	public IMiniBagPairsIterator<K, V> implBagIterator() {
		Set<Map.Entry<K,IMiniBagSingles<V>>> set = adaptee.entrySet();
		Iterator<Map.Entry<K,IMiniBagSingles<V>>> it = set.iterator();
		Map.Entry<K,IMiniBagSingles<V>> currentEntry = null;
    	if (it.hasNext()) currentEntry = it.next();
    	return new ImplJCBagPairsIteratorAdaptor<K,V>(it, currentEntry);
	}

	public boolean implContainsPair(K key, V value) {
		boolean contains = false;
		IMiniBagSingles<V> values = adaptee.get(key);
		if (values != null) {
			contains = values.contains(value);
		}
		return contains;
	}

	
	public int implCountPair(K key, V value) {
		Integer count = 0;
		IMiniBagSingles<V> values = adaptee.get(key);
		if (values != null) {
			count = values.count(value);
		}
		return count;
	}

	
	public IMiniSequence<IMiniKeyValuePair<IMiniKeyValuePair<K, V>, Integer>> 
					implGetKeyValuePairsAndCounts() {
		IFactoryMiniSequence<IMiniKeyValuePair<IMiniKeyValuePair<K, V>, Integer>> factory 
			= new FactoryMiniSequence<IMiniKeyValuePair<IMiniKeyValuePair<K, V>, Integer>>();
		IMiniSequence<IMiniKeyValuePair<IMiniKeyValuePair<K, V>, Integer>> 
						pairsSequence = factory.makeSequence();
		Set<Map.Entry<K, IMiniBagSingles<V>>> pairs = adaptee.entrySet();
		for (Map.Entry<K, IMiniBagSingles<V>> keyValuePair : pairs) {
			for(IMiniKeyValuePair<V,Integer> valuePair : keyValuePair.getValue()) {
				pairsSequence.add(new MiniKeyValuePair<IMiniKeyValuePair<K, V>, Integer>(
						new MiniKeyValuePair<K, V>(keyValuePair.getKey(),valuePair.getFirst())
						,valuePair.getSecond()));
			}
		}
		return pairsSequence;
	}

	
	public IMiniSequence<IMiniKeyValuePair<K, V>> implGetKeyValuePairs() {
		IFactoryMiniSequence<IMiniKeyValuePair<K, V>> factory 
			= new FactoryMiniSequence<IMiniKeyValuePair<K, V>>();
		IMiniSequence<IMiniKeyValuePair<K, V>> 
						pairsSequence = factory.makeSequence();
		Set<Map.Entry<K, IMiniBagSingles<V>>> pairs = adaptee.entrySet();
		for (Map.Entry<K, IMiniBagSingles<V>> keyValuePair : pairs) {
			for(IMiniKeyValuePair<V,Integer> valuePair : keyValuePair.getValue()) {
				pairsSequence.add(new MiniKeyValuePair<K, V>(keyValuePair.getKey(),valuePair.getFirst()));
			}
		}
		return pairsSequence;
	}
	
	public IMiniSequence<K> implGetKeys() {
		IFactoryMiniSequence<K> factory 
				= new FactoryMiniSequence<K>();
		IMiniSequence<K> 
					keys = factory.makeSequence();
		for (K k : adaptee.keySet()) { keys.add(k); }
		return keys;
	}
	
	
	public IMiniSequence<IMiniKeyValuePair<V,Integer>> 
					implGetValuesAndCounts(K key) {
		IMiniSequence<IMiniKeyValuePair<V,Integer>> pairs = null;
		IMiniBagSingles<V> valuePairs = adaptee.get(key);
		if (valuePairs != null) {
			IFactoryMiniSequence<IMiniKeyValuePair<V, Integer>> factory 
				= new FactoryMiniSequence<IMiniKeyValuePair<V, Integer>>();
			pairs = factory.makeSequence();
			for (IMiniKeyValuePair<V,Integer> pair : valuePairs) {
				pairs.add(pair);
			}
		}
		return pairs;
	}
	

	public IMiniSequence<V> implGetValues(K key) {
		IMiniSequence<V> values = null;
		IMiniBagSingles<V> valuePairs = adaptee.get(key);
		if (valuePairs != null ) {
			values = valuePairs.getKeys();
		
		}
		return values;
	}

	public boolean implRemove(K key) {
		adaptee.remove(key);
		return adaptee.containsKey(key);

	}

	
	public boolean implRemoveOnePair(K key, V value) {
		boolean removed = false;
		IMiniBagSingles<V> valuePairs = adaptee.get(key);
		if (valuePairs != null) {
			removed = valuePairs.removeOne(value);
		}
		if (valuePairs.isEmpty()) {
			implRemove(key);
		}
		return removed;
	}
	
	public boolean implRemoveAllPair(K key, V value) {
		boolean removed = false;
		IMiniBagSingles<V> valuePairs = adaptee.get(key);
		if (valuePairs != null) {
			removed = valuePairs.removeAll(value);
		}
		if (valuePairs.isEmpty()) {
			implRemove(key);
		}
		return removed;
	}
	
		
	// not great = arbitrarily sets mapping to be 'first' mapping
	public void implRemoveDuplicateKeys() {
		// this would mean setting all the keys to 1!
		Set<Map.Entry<K, IMiniBagSingles<V>>> pairs = adaptee.entrySet();
		for (Map.Entry<K, IMiniBagSingles<V>> pair : pairs) {
			V v = null;
			if (pair.getValue().uniqueKeySize() > 0) {
				v = pair.getValue().bagIterator().currentKey();
			}
			pair.getValue().clear();
			pair.getValue().add(v);
		}
	}
	
}
