package minicollection;

import java.util.*;

public abstract class AbstractImplJCBagSinglesAdaptor<K> implements
		IMiniBagSinglesImplementation<K> {

	Map<K,Integer> adaptee;
	
		
	// default implementation
	public AbstractImplJCBagSinglesAdaptor(Map<K,Integer> adaptee) {
		this.adaptee = adaptee;
	}
	
	public int implSize() {
		// number of keys * counts
		//return adaptee.size();
		List<Integer> values = new ArrayList<Integer>(adaptee.values());
		Integer count = 0;
		for (Integer v : values) {count += v; }
		return count;
	}
	
	public boolean implContains(Object key) {
		return adaptee.containsKey(key);
	}

	public int implCount(Object key) {
		int count = 0;
		Integer countInteger = adaptee.get(key);
		if (countInteger != null) count = countInteger;
		return count;
	}

	public boolean implIsEmpty() {
		return adaptee.isEmpty();
	}
	
	public void implRemoveDuplicates() {
		// this would mean setting all the keys to 1!
		for (K k : adaptee.keySet()) {adaptee.put(k, 1);}
	}

	public boolean implClear() {
		adaptee.clear();
		return adaptee.isEmpty();
	}

	
	public Iterator<IMiniKeyValuePair<K,Integer>> implIterator() {
		return new ImplJCIteratorForBagSinglesAdaptor<K>(
				implBagIterator());
	}
	
	public IMiniIterator<IMiniKeyValuePair<K,Integer>> implMiniIterator() {
		return implBagIterator();
				
	}
	
	public int implUniqueKeySize() {
		// number of keys
		return adaptee.size();
	}
	
	public InsertCheck implAdd(K key) {
		Integer count = adaptee.get(key);
		
		if (count == null) { // not already there
			adaptee.put(key, 1);
		}
		else adaptee.put(key, count+1);
		return (adaptee.containsKey(key) ?
						InsertCheck.NEW : InsertCheck.FAILED);
		}

	public boolean implRemoveOne(K key) {
		boolean removed = false;
		Integer count = adaptee.get(key);
		if (count != null) {
			removed = true;
			count--;
		
			if (count > 0) adaptee.put(key, count);
			else adaptee.remove(key);
		}
		return removed;
	}
	
	public boolean implRemoveAll(K key) {
		boolean removed = false;
		Integer count = adaptee.get(key);
		if (count != null) {
			adaptee.remove(key);
			removed = true;
		}
		return removed;
	}
	
	public IMiniSequence<K> implGetKeys() {
		IFactoryMiniSequence<K> factory = 
			new FactoryMiniSequence<K>();
		
		IMiniSequence<K> seq = factory.makeSequence();
		for (K k : adaptee.keySet()) {
			seq.add(k);
		}
		return seq;
	}
	
	public IMiniSequence<IMiniKeyValuePair<K, Integer>> implGetKeysAndCounts() {
		
		
		IFactoryMiniSequence<IMiniKeyValuePair<K, Integer>> factory = 
			new FactoryMiniSequence<IMiniKeyValuePair<K, Integer>>();
		
		IMiniSequence<IMiniKeyValuePair<K, Integer>> seq = factory.makeSequence();
		
		for (K k : adaptee.keySet()) {
			seq.add(new MiniKeyValuePair<K,Integer>(k,adaptee.get(k)));
		}
		
		return seq;
	}

	
	public IMiniBagSinglesIterator<K> implBagIterator() {
		Set<Map.Entry<K,Integer>> set = adaptee.entrySet();
		Iterator<Map.Entry<K,Integer>> it = set.iterator();
		Map.Entry<K,Integer> currentEntry = null;
    	if (it.hasNext()) currentEntry = it.next();
    	return new ImplJCBagSinglesIteratorAdaptor<K>(it, currentEntry);
	}


}
