package minicollection;

import java.util.*;

public abstract class AbstractImplJCSequenceAdaptor<E> 
					implements IMiniSequenceImplementation<E>  {
	
	List<E> adaptee;
	
	AccessMode accessMode = AccessMode.RANDOM;
	
	// default implementation
	AbstractImplJCSequenceAdaptor(List<E> adaptee, 
			AccessMode accessMode) {
		this.adaptee = adaptee;
		this.accessMode = accessMode;
	}
	
	public AccessMode implGetAccessMode() {return accessMode; }
	
	public int implSize() { return adaptee.size(); }
    
	public boolean implIsEmpty() { return adaptee.isEmpty(); }
    
	public boolean implContains(Object element) { return adaptee.contains(element); }
    
	public int implCount(Object element) {return Collections.frequency(adaptee, element); }
    
	public boolean implClear() {
		adaptee.clear();
		return adaptee.isEmpty();
	}
    
    // Iteration
    //INewIterator<E> newiterator(); // from extends INewIterable
    //Iterator<E> iterator(); // from extends Iterable

    public InsertCheck implAdd(E element) { 
    	InsertCheck added = (adaptee.add(element)) 
    						? InsertCheck.NEW : InsertCheck.FAILED;
    	return added;
    }
    
    public boolean implRemoveAll(E element) {
    	while (adaptee.contains(element)) {
    		adaptee.remove(element);
    	}
    	return adaptee.contains(element);
    }
    
    public void implRemoveDuplicates() {
    	Set<E> tempset= new LinkedHashSet<E>(adaptee);
    	adaptee = new ArrayList<E>(tempset);
    }

    @SuppressWarnings("unchecked")
    public void implSort() {
    	//implSort(new CustomComparator<E>());
    	//Collections.sort(adaptee, new CustomComparator<E>());
    
    	Collections.sort(adaptee, new CustomComparator());
    }
    
    public void implSort(Comparator <? super E> c) {
    	Collections.sort(adaptee, c);
    }
    
	public E implGetAtIndex(int index) {return adaptee.get(index); }
    
	public boolean implResetAtIndex(int index, E element) { 
    	adaptee.set(index, element);
    	return adaptee.get(index) == element;
    } 
	
	
	// makes no sense - see iterator add - should be private or something?
    public InsertCheck implAddAtIndex(int index, E element) {
    	adaptee.add(index, element);
    	InsertCheck added = (adaptee.get(index) == element) 
    						? InsertCheck.NEW : InsertCheck.FAILED;
    	return added;
    }
    
    public E implRemoveAtIndex(int index) {return adaptee.remove(index); } 
    
    //boolean addAll(int index, Collection<? extends E> c); //optional

    // Search
	public int implFirstIndexOf(Object o) { return adaptee.indexOf(o); }
	
    public int implNextIndexOf(int index, Object o ) {
    	int retIndex = -1;
    	if (adaptee.lastIndexOf(o) >= index) {
	    	List<E> sub = adaptee.subList(index, adaptee.lastIndexOf(o)+1);
	    	retIndex = index + sub.indexOf(o);
    	}
    	return retIndex;
    }
    
    public int implLastIndexOf(Object o)  { return adaptee.lastIndexOf(o); }
    
    // Iteration
    public IMiniSequenceIterator<E> implBegin() { // first element
    	ListIterator<E> it = adaptee.listIterator();
    	E current = null;
    	if (it.hasNext()) current = it.next();
    	return new ImplJCSequenceIteratorAdaptor<E>(it, current);
    }
    
    public IMiniSequenceIterator<E> implEnd() {  // last element
    	ListIterator<E> it = adaptee.listIterator(adaptee.size()-1);
    	E current = null;
    	if (it.hasNext()) current = it.next();
    	it.previous();
    	return new ImplJCSequenceIteratorAdaptor<E>(it, current);
    }
    
    public IMiniSequenceIterator<E> implSeqIterator(int index) {
    	ListIterator<E> it = adaptee.listIterator(index);
    	E current = null;
    	if (it.hasNext()) current = it.next();
    	return new ImplJCSequenceIteratorAdaptor<E>(it, current);
    }
    
    public IMiniIterator<E> implMiniIterator() {
    	ListIterator<E> it = adaptee.listIterator();
    	E current = null;
    	if (it.hasNext()) current = it.next();
    	return new ImplJCSequenceIteratorAdaptor<E>(it, current);
    }
    
    public Iterator<E> implIterator() {
    	return new ImplJCIteratorForSequenceAdaptor<E>(
				implBegin());
    	//return adaptee.iterator();
    }
    
     
    private static class CustomComparator<E extends Comparable<? super E>>
	implements Comparator<E>
	{
		public int compare(E e1, E e2) {
			
			return e1.compareTo(e2);
		}
	}
    
    
    // Range-view
    //List<E> subList(int from, int to);



}
