package minicollection;

public abstract class AbstractMiniPair<E, T> 
								implements IMiniPair<E, T> {
	
	//datamembers
	E first;
	T second;
	
	AbstractMiniPair(E first, T second) {
		this.first = first;
		this.second = second;
	}
			
	public E getFirst() { return first;}
	
	public T getSecond() { return second;}
	
	// overridable implementation of methods from interface
	public boolean equals(Object o) {
		boolean areEqual = false;
		
		if (o == this) areEqual = true;
		
		// equals with collections objects 
		// see Java Generics and Collections, pg 78-79
		// they use instanceof and I think that this makes sense in the collections context
		else if (o != null && getClass() == o.getClass()) { 
			// check the contents
			Object oFirst = ((IMiniPair<?,?>)o).getFirst();
			Object oSecond = ((IMiniPair<?,?>)o).getSecond();
			
			boolean firstsEqual = (first == null ? oFirst == null : first.equals(oFirst));
			boolean secondsEqual = (second == null ? oSecond == null : second.equals(oFirst));
			
			areEqual = (firstsEqual && secondsEqual);
		}
		//else areEqual will be null
		return areEqual;
	}
			
	public int hashCode() {
		return first.hashCode()*33 + second.hashCode();
	}
	
	public String toString() {
		return "(" + first.toString() + ", " + second.toString() + ")"; }

}
