package minicollection;

import java.util.Comparator;
import java.util.Iterator;

public abstract class AbstractMiniSequenceBridge<E> 
			extends AbstractMiniSequence<E> {
	
	//implementation
	private IMiniSequenceImplementation<E> impl;
	
	//constructor
	AbstractMiniSequenceBridge(IMiniSequenceImplementation<E> impl,
						MembershipMode membershipMode, 
						OrderingMode orderingMode) {
		super(membershipMode, orderingMode);
		this.impl = impl;
	}
	
	public AccessMode getAccessMode() {
		return impl.implGetAccessMode();
	}
			
	public int count(Object element) {
		return impl.implCount(element);
	}

	public boolean isEmpty() {
		return impl.implIsEmpty();
	}

	public int size() {
		return impl.implSize();
	}
	
	public boolean contains(Object element) {
		return impl.implContains(element);
	}
	
	public InsertCheck add(E element) { 
		return impl.implAdd(element); }

    public boolean clear() { return impl.implClear(); }

	public boolean removeAll(E element) {
		return impl.implRemoveAll(element);
	}
	
	public void removeDuplicates() {
		impl.implRemoveDuplicates();
	}
	
	public IMiniIterator<E> miniIterator() {
		return impl.implMiniIterator();
	}
	
	public Iterator<E> iterator() {
		return impl.implIterator();
	}
	
	protected void primitive_sort() {
		impl.implSort();
	}
	
	protected void primitive_sort( Comparator<? super E> c) {
		impl.implSort(c);
	}

    protected E primitive_getAtIndex(int index) {
    	return impl.implGetAtIndex(index);
    }
    
    //boolean addAll(int index, Collection<? extends E> c); //optional
    
    // Search
    public int firstIndexof(Object o) { return impl.implFirstIndexOf(o); }
    
    protected int primitive_nextIndexOf(int index, Object o ) {
    	return impl.implNextIndexOf(index, o); }
        
    public int lastIndexOf(Object o) { return impl.implLastIndexOf(o); }
    
    public int firstIndexOf(Object o) {
		return impl.implFirstIndexOf(o);
	}
    
    /*
	 protected boolean primitive_resetAtIndex(int index, E element) {
    	return impl.implResetAtIndex(index, element);
    }
    
    protected InsertCheck primitive_addAtIndex(int index, E element) {
    	return impl.implAddAtIndex(index, element);
    }
    
    
    protected E primitive_removeAtIndex(int index) { 
    	return impl.implRemoveAtIndex(index); }
    */
    
    // Iteration
    // check the modification status before returning the iterator
    public IMiniSequenceIterator<E> begin() { return impl.implBegin(); }
    
    public IMiniSequenceIterator<E> end() { return impl.implEnd(); }
    
    protected IMiniSequenceIterator<E> primitive_seqIterator(int index) {
    	return impl.implSeqIterator(index);
    }
	
	protected void primitive_resetAccessMode(AccessMode accessMode) {
		IFactoryAdaptorMiniSequence<E> adaptorFactory = 
	    		new FactoryJCAdaptorMiniSeqence<E>();
	    	this.impl = adaptorFactory.makeSequenceAdaptor(this, accessMode);
	}
	
	protected IMiniSequenceImplementation<E> getImpl()
		{ return impl; }
		
}
    