package minicollection;


public class FactoryJCAdaptorMiniSeqence<E> 
		implements IFactoryAdaptorMiniSequence<E> {
	
	
	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactoryNew#makeSequenceAdaptor()
	 */
	public IMiniSequenceImplementation<E> makeSequenceAdaptor() {
		return makeRandomAccessAdaptor();
		
	}
	
	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactoryNew#makeSequenceAdaptor(minicollection.IMiniCollection)
	 */
	public IMiniSequenceImplementation<E> makeSequenceAdaptor(
			IMiniCollection<? extends E> anotherCollection) {
		return makeRandomAccessAdaptor(anotherCollection);
	}

	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactoryNew#makeSequenceAdaptor(E, int)
	 */
	public IMiniSequenceImplementation<E> makeSequenceAdaptor(
			E e, int n) {
		return  makeRandomAccessAdaptor(e, n);
	}
	
	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactoryNew#makeSequenceAdaptor(minicollection.AccessMode)
	 */
	public IMiniSequenceImplementation<E> makeSequenceAdaptor(
							AccessMode accessMode) {
		IMiniSequenceImplementation<E> impl = null;
		if (accessMode == AccessMode.RANDOM) {
			impl = makeRandomAccessAdaptor();
		}
		else if (accessMode == AccessMode.SEQUENTIAL) {
			impl = makeSequentialAccessAdaptor();
		}
		else {
			throw new UnsupportedOperationException(
					"Sorry, I don't know that access mode");
		}
		return impl;
	}
	
	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactoryNew#makeSequenceAdaptor(minicollection.IMiniCollection, minicollection.AccessMode)
	 */
	public IMiniSequenceImplementation<E> makeSequenceAdaptor(
						IMiniCollection<? extends E> anotherCollection,
						AccessMode accessMode) {
		IMiniSequenceImplementation<E> impl = null;
		if (accessMode == AccessMode.RANDOM) {
			impl = makeRandomAccessAdaptor(anotherCollection);
		}
		else if (accessMode == AccessMode.SEQUENTIAL) {
			impl = makeSequentialAccessAdaptor(anotherCollection);
		}
		else {
			throw new UnsupportedOperationException(
					"Sorry, I don't know that access mode");
		}
		return impl;
	}
	
	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactoryNew#makeSequenceAdaptor(E, int, minicollection.AccessMode)
	 */
	public IMiniSequenceImplementation<E> makeSequenceAdaptor(
			E e, int n, AccessMode accessMode) {
		IMiniSequenceImplementation<E> impl = null;
		if (accessMode == AccessMode.RANDOM) {
		impl = makeRandomAccessAdaptor(e, n);
		}
		else if (accessMode == AccessMode.SEQUENTIAL) {
		impl = makeSequentialAccessAdaptor(e, n);
		}
		else {
		throw new UnsupportedOperationException(
				"Sorry, I don't know that access mode");
		}
		return impl;
	}
	
	
	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactory#makeRandomAccessAdaptor()
	 */
	private IMiniSequenceImplementation<E> makeRandomAccessAdaptor() {
		return new ImplJCSequenceRandomAdaptor<E>(); 
	}
	
	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactory#makeRandomAccessAdaptor(minicollection.IMiniCollection)
	 */
	private IMiniSequenceImplementation<E> makeRandomAccessAdaptor
		(IMiniCollection<? extends E> anotherCollection) {
		IMiniSequenceImplementation<E> adaptor = 
			new ImplJCSequenceRandomAdaptor<E>();
		for (E e : anotherCollection) { adaptor.implAdd(e); }
		return adaptor;
	}
	
	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactory#makeRandomAccessAdaptor(E, int)
	 */
	private IMiniSequenceImplementation<E> makeRandomAccessAdaptor
	(E e, int n) {
	IMiniSequenceImplementation<E> adaptor = 
		new ImplJCSequenceRandomAdaptor<E>();
	while (n>0) { 
		adaptor.implAdd(e);
		n--;}
	return adaptor;
	}

	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactory#makeSequentialAccessAdaptor()
	 */
	private IMiniSequenceImplementation<E> makeSequentialAccessAdaptor() {
		return new ImplJCSequenceSequentialAdaptor<E>(); 
	}
	
	
	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactory#makeSequentialAccessAdaptor(minicollection.IMiniCollection)
	 */
	private IMiniSequenceImplementation<E> makeSequentialAccessAdaptor
	(IMiniCollection<? extends E> anotherCollection) {
		IMiniSequenceImplementation<E> adaptor = 
			new ImplJCSequenceSequentialAdaptor<E>();
		for (E e : anotherCollection) { adaptor.implAdd(e); }
		return adaptor;
	}

	/* (non-Javadoc)
	 * @see minicollection.IMiniCollectionAdaptorFactory#makeSequentialAccessAdaptor(E, int)
	 */
	private IMiniSequenceImplementation<E> makeSequentialAccessAdaptor
	(E e, int n) {
		IMiniSequenceImplementation<E> adaptor = 
			new ImplJCSequenceSequentialAdaptor<E>();
		while (n>0) { 
			adaptor.implAdd(e);
			n--;}
		return adaptor;
	}
	
	
	
}
