package minicollection;

import java.util.Comparator;

public class FactoryMiniSequenceCustomSorted<E>
	extends AbstractFactoryMiniCollection<E> 
	implements IFactoryMiniSequence<E> {

	private Comparator<? super E> comp;
	private OrderingMode orderingMode;
	private IFactoryMiniSequence<E> sequenceFactory = 
		new FactoryMiniSequence<E>();
	
	
	private FactoryMiniSequenceCustomSorted() {};
	
	private FactoryMiniSequenceCustomSorted(
			IFactoryAdaptorMiniSequence<E> adaptorFactory) {
		sequenceFactory = new FactoryMiniSequence<E>(adaptorFactory);
	};
	
	public FactoryMiniSequenceCustomSorted(Comparator<? super E> comp) {
		this.comp = comp;
		this.orderingMode = OrderingMode.CUSTOM_ORDERING;
	}
	
	public FactoryMiniSequenceCustomSorted(Comparator<? super E> comp,
			IFactoryAdaptorMiniSequence<E> adaptorFactory) {
		this.comp = comp;
		this.orderingMode = OrderingMode.CUSTOM_ORDERING;
		this.sequenceFactory = new FactoryMiniSequence<E>(adaptorFactory);
	}
	
	protected FactoryMiniSequenceCustomSorted(Comparator<? super E> comp,
					OrderingMode orderingMode) {
		this.comp = comp;
		this.orderingMode = OrderingMode.NATURAL_ORDERING;
	}
	
	protected FactoryMiniSequenceCustomSorted(Comparator<? super E> comp,
			OrderingMode orderingMode,
			IFactoryAdaptorMiniSequence<E> adaptorFactory) {
		this.comp = comp;
		this.orderingMode = OrderingMode.NATURAL_ORDERING;
		this.sequenceFactory = new FactoryMiniSequence<E>(adaptorFactory);
	}
	
	
	public IMiniSequence<E> makeSequence() {
		return new MiniSequenceCustomSortedDecorator<E>(
				sequenceFactory.makeSequence(), 
				comp, orderingMode);
	}
	
	public IMiniSequence<E> makeSequence(AccessMode accessMode) {
		return new MiniSequenceCustomSortedDecorator<E>(
				sequenceFactory.makeSequence(accessMode), 
				comp, orderingMode);
	}
	
	public IMiniSequence<E> makeSequence(
			IMiniCollection<? extends E> anotherCollection) {
		return new MiniSequenceCustomSortedDecorator<E>(
				sequenceFactory.makeSequence(anotherCollection), 
				comp, orderingMode);
	}
	
	public IMiniSequence<E> makeSequence(
			IMiniCollection<? extends E> anotherCollection,
			AccessMode accessMode) {
		return new MiniSequenceCustomSortedDecorator<E>(
				sequenceFactory.makeSequence(anotherCollection, accessMode), 
				comp, orderingMode);
	}
	
	public IMiniSequence<E> makeSequence(E e, int n) {
		return new MiniSequenceCustomSortedDecorator<E>(
				sequenceFactory.makeSequence(e, n), 
				comp, orderingMode);
	}
	
	public IMiniSequence<E> makeSequence(E e, int n, AccessMode accessMode) {

		return new MiniSequenceCustomSortedDecorator<E>(
				sequenceFactory.makeSequence(e, n, accessMode), 
				comp, orderingMode);
	}
	
}
