package minicollection;


/**
 * A collection type for elements of type E where each element has an
 * ordered position or index in the collection.
 * 
 * @author jah217
 *
 */

public interface IMiniSequenceFlexiAdd<E> 
					extends IMiniSequence<E> {
	
	
    
	/**
	 * Reset the element at the specified index in the sequence to a specified
	 * element.
	 * @throws an IndexOutOfBoundsException if index < 0 or index >= size()
	 * @pre 0 <= index < size()
	 * @post getAtIndex(index) == element
	 * @param index the index value of the element to be reset
	 * @param element the value of the element to be at the specified index
	 * value in the sequence
	 * @return true when the operation is successfully completed
	 */
	boolean resetAtIndex(int index, E element); 
    
	/**
	 * Add an element to the the sequence at a specified index value.
	 * The sequence should attempt to add the element to the elements it 
	 * contains at the specified index value and change the indexes of 
	 * pre-existing elements in the collection to reflect the addition.  
	 * The operation should return an enumerated type specifying if and 
	 * how the element was added to the sequence.  
	 * The operation may return FAILED if and only if the attempt to add 
	 * the element fails for reasons beyond the control of the sequence.
	 * If the addition of the element results in the replacement of any
	 * pre-existing member of the collection with the element, the 
	 * operation should return OVERWRITTEN.  If the addition of the element
	 * does not result in the replacement of a previously existing member
	 * of the collection with the element the operation should return NEW.
	 * The operation may result in changes in index values of pre-existing
	 * members of the collection to accommodate the addition of the element.
	 *
	 * @throws an IndexOutOfBoundsException if index < 0 or index > size()
	 * @pre 0 <= index <= size().
	 * @post	- getAtIndex(index) == element
	 *		 	- and if the operation returns NEW then the element has been added to 
	 * the the sequence and all members of the sequence prior to the inception
	 * of the operation are still in the sequence.  If the operation returns
	 * OVERWRITTEN then the element has replaced an element e in the sequence
	 * which was present at the inception of the operation and for which 
	 * e.equals(element) == true.
	 * 			- and there is no change in the ordering mode of the collection.  
	 * @param index the index value at which the element is to be added
	 * @param element the value of the element to be added at the specified 
	 * index value in the sequence.
	 * @return an InsertCheck value reflecting the result of the operation.
	 */
	InsertCheck addAtIndex(int index, E element);
	
	/**
	 * Remove the element at the specified index in the sequence and change the 
	 * index values of the remaining elements in the sequence to reflect the 
	 * removal.
	 * @throws an IndexOutOfBoundsException if index < 0 or index >= size()
	 * @pre 0 <= index < size()
	 * @post the object which was at the specified index value is no longer
	 * in the sequence (but the sequence may still contain duplicate copies of that
	 * object at other index values) and the size of the sequence has reduced by 1.
	 * @param index the index value of the element to be removed
	 * @return the element with the specified index value at the inception of 
	 * the operation. 
	 */
	E removeAtIndex(int index);
   
}
