package minicollection;



/**
 * @author jah217
 *
 * @param <E>
 */
public class MiniSequenceFlexiAddUniqueOnlyDecorator<E> 
				extends AbstractMiniSequenceFlexiAddDecorator<E> {
	
	
	public MiniSequenceFlexiAddUniqueOnlyDecorator(IMiniSequenceFlexiAdd<E> inner) {
		// true for custom sorted
		super(inner, MembershipMode.UNIQUES_ONLY, 
				inner.getOrderingMode());
		inner.removeDuplicates();
	}
	
	
	@Override
	public InsertCheck add(E element) {
		InsertCheck isAdded = InsertCheck.FAILED;
		int index = inner.firstIndexOf(element);
		if (index < 0) isAdded = inner.add(element);
		else 
		{
			IFactoryMiniSequence<E> factory = new FactoryMiniSequence<E>();
			IMiniSequence<E> seq = factory.makeSequence();
			for (int i = size()-1; i > index; i--) {
				seq.add(removeAtIndex(i));
				
			}
			removeAtIndex(index);
			if (add(element) != InsertCheck.FAILED)
				isAdded = InsertCheck.OVERWRITTEN;
			for (int i = seq.size()-1; i >= 0; i--) {
				add(seq.getAtIndex(i));
				
			}
		}
		return isAdded;
	}
	
	// add if it is not there, or overwrite if it is there at the 
	// specified index, or shuffle the current elements to overwrite
	// first occurrence it is already there at a different index
	@Override
	protected InsertCheck primitive_addAtIndex(int index, E element) {
		int i = inner.firstIndexOf(element);
		InsertCheck isAdded = InsertCheck.FAILED;
		if (i < 0) isAdded = inner.addAtIndex(index, element);
		else if ( i == index) { // overwrite
			if (inner.resetAtIndex(index, element)) 
				isAdded = InsertCheck.OVERWRITTEN;
			}
		
		else if (i < index) { // need to shuffle elements
			E in = element;
			int j = index;
			while (j >= i) {
				E out = inner.getAtIndex(j);
				inner.resetAtIndex(j, in);
				in = out;
				j--;
				isAdded = InsertCheck.OVERWRITTEN;
			} // the first occurrence of element will be overwritten
		}
		
		else if (i > index) { // need to shuffle elements
			E in = element;
			int j = index;
			while (j <= i) {
				E out = inner.getAtIndex(j);
				inner.resetAtIndex(j, in);
				in = out;
				j++;
				isAdded = InsertCheck.OVERWRITTEN;
			} // the first occurrence of element will be overwritten
		}
		return isAdded;
	}
	
	@Override
	protected boolean primitive_resetAtIndex(int index, E element) {
		int i = inner.firstIndexOf(element);
		boolean isReset = inner.resetAtIndex(index, element);
		if (i >= 0 && i != index) { // put it here and remove the other
			inner.removeAtIndex(i);
		}
		return isReset;
	}
	
}
