package testing;

import minicollection.*;

/**
 * @author Jenny Harlow
 * 
 * Test the IMiniBagSinglesIterator
 *
 */
public class TestBagSinglesIterator {
	
	

	public static void main(String[] args) {
		
		IFactoryMiniBagSingles<String> factory = 
			new FactoryMiniBagSingles<String>();
		
		IMiniBagSingles<String> bag = factory.makeBagSingles();
		System.out.println("made empty bag");
		
		System.out.println("checking configuration");
		System.out.println("getMembershipMode() = " + bag.getMembershipMode());
		System.out.println();
		
		
		System.out.println("IsEmpty is " + bag.isEmpty());
		System.out.println("Size is " + bag.size());
		
		bag.add("Jenny");
		bag.add("Wal");
		bag.add("Linda");
		
		System.out.println("added Jenny, Wal, Linda");
		System.out.println("IsEmpty is " + bag.isEmpty());
		System.out.println("Size is " + bag.size());
		
		try {
			System.out.println("iterating using for each");
			for (IMiniKeyValuePair<String,Integer> pair : bag )
				System.out.print(pair + "  ");
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.getMessage());
		}
		System.out.println();
		try {
			System.out.println("iterating using the MiniIterator, which gives pairs form");
			IMiniIterator<IMiniKeyValuePair<String,Integer>> it;
			it = bag.miniIterator();
			while(it.current() != null) {
				System.out.print(it.current() + "  ");
				it.next();
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.getMessage());
		}
		System.out.println();
		
		try {
			System.out.println("iterating using the BagSinglesIterator, in pairs form");
			IMiniBagSinglesIterator<String> it = bag.bagIterator();
			while(it.current() != null) {
				System.out.print(it.current() + "  ");
				it.next();
			}
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		
		try {
			System.out.println("iterating using the BagSinglesMiniIterator, " +
					"using currentKey() and currentCount()");
			IMiniBagSinglesIterator<String> it = bag.bagIterator();
			while(it.current() != null) {
				System.out.print("key " + it.currentKey() + " has count " + it.currentCount() + "   ");
				it.next();
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.getMessage());
		}
		System.out.println();
		
		
		System.out.println();
		try {	
			IMiniBagSinglesIterator<String> it  = bag.bagIterator();
			IMiniKeyValuePair<String, Integer> currentElement 
					= it.current();
			System.out.println("it  = bag.begin(), current is " + currentElement);
			String currentElementKey = currentElement.getFirst();
			
			System.out.println("now trying removeCurrent: ");
			it.removeCurrent();
			System.out.println("it.current() == null is " 
					+ (it.current() == null));
			it.next();
			System.out.println("after calling it.next(), it.current() = " 
					+ it.current());
			System.out.println("put " + currentElementKey + " back in, bag now is");
			bag.add(currentElementKey);
			for (IMiniKeyValuePair<String, Integer> pair : bag) { System.out.print(pair + "  ");}
			System.out.println();
			
			System.out.println("try calling it.next() on the same iterator (this should give an exception)");
			try {
				it.next();
				
			}
			catch (Exception e) {
				System.out.println("That resulted in an exception:");
				System.out.println(e.toString());
				
			}
			System.out.println();
					
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
		System.out.println();
		
		try {	
			System.out.println("try to use for each to remove everything (this should give an exception)");
			for (IMiniKeyValuePair<String,Integer> pair : bag) {
				bag.removeOne(pair.getFirst());
			}
			
			System.out.println("size now is " + bag.size());
			System.out.println();
			
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
	
		System.out.println();
		try {	
						
			System.out.println("try to use iteration on MiniIterator to remove everything");
			IMiniBagSinglesIterator<String> it = bag.bagIterator();
			while(it.current() != null) {
				System.out.println("removing " + it.current());
				it.removeCurrent();
				it.next();
			}
			System.out.println("size now is " + bag.size());
			System.out.println();
			
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
		System.out.println();
		try {	
			bag.add("Jenny");
			bag.add("Wal");
			bag.add("Linda");
			System.out.println("put three more in, sequence in pairs form now is");
			for (IMiniKeyValuePair<String,Integer> pair : bag) { System.out.print(pair + "  ");}
			System.out.println();
			System.out.println("and sequence using just keys is");
			for (IMiniKeyValuePair<String,Integer> pair : bag) { System.out.print(pair.getFirst() + "  ");}
			System.out.println();
			System.out.println("try to use iteration on MiniBagIterator to remove everything");
			IMiniBagSinglesIterator<String> it = bag.bagIterator();
			while(it.current() != null) {
				System.out.println("removing " + it.current());
				it.removeCurrent();
				it.next();
			}
			System.out.println("size now is " + bag.size());
			System.out.println();
			
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
		
		
	}

}
