package testing;



import minicollection.*;


/**
 * @author Jenny Harlow
 * 
 * Test miniIterator() for bags of pairs
 *
 */
public class TestBasicBagIterator {

	public static void main(String[] args) {
		
		IMiniBagPairs<Author, CharacterType> bag = null;
		
		IFactoryMiniBagPairs<Author, CharacterType> factory = 
			new FactoryMiniBagPairs<Author,CharacterType>();
		
		Author Heyer = new Author("Georgette Heyer");
		Author Simenon = new Author("Simenon");
		Author Pratchett = new Author("Terry Pratchett");
		Author Fleming = new Author("Ian Fleming");
		
		CharacterType StrongAndSilent = new CharacterType("Strong and Silent");
		CharacterType SAndS = new CharacterType("Strong and Silent");
		CharacterType IntelligentHeroine = new CharacterType("Intelligent Heroine");
		CharacterType FluffyBlonde = new CharacterType("Fluffy Blonde");
		CharacterType BitOfStuff = new CharacterType("Fluffy Blonde");
		CharacterType HeavyPoliceman = new CharacterType("Heavy Policeman");
		CharacterType DowntroddenWife = new CharacterType("Downtrodden Wife");
		CharacterType HenpeckedHusband = new CharacterType("Henpecked Husband");
		CharacterType EvilVillain = new CharacterType("Evil Villain");
		CharacterType MaleFriend = new CharacterType("Uncompetitive Male Friend");
		CharacterType Wizard = new CharacterType("Wizard");
		CharacterType Witch = new CharacterType("Witch");
		CharacterType Troll = new CharacterType("Troll");
		CharacterType Death = new CharacterType("Death");
		CharacterType Nemesis = new CharacterType("Death");
		CharacterType Retribution = new CharacterType("Death");
		
		try {
			bag = factory.makeBagPairs();
			
			System.out.println("Made a bag: checking size and configuration");
			System.out.println("getMembershipMode() = " + bag.getMembershipMode());
			System.out.println("IsEmpty() is " + bag.isEmpty());
			System.out.println("Size() is " + bag.size());
			System.out.println();
			
			for (int i = 0; i < 5 ; i++) {	bag.add(Heyer, StrongAndSilent); }
			for (int i = 0; i < 6 ; i++) {	bag.add(Heyer, IntelligentHeroine); }
			
			for (int i = 0; i < 2 ; i++) {	bag.add(Simenon, DowntroddenWife); }
			for (int i = 0; i < 1 ; i++) {	bag.add(Simenon, HenpeckedHusband); }
			for (int i = 0; i < 3 ; i++) {	bag.add(Simenon, Nemesis); }
			for (int i = 0; i < 10 ; i++) {	bag.add(Pratchett, Wizard); }
			for (int i = 0; i < 4 ; i++) {	bag.add(Pratchett, Witch); }
			for (int i = 0; i < 1 ; i++) {	bag.add(Pratchett, Death); }
			for (int i = 0; i < 1 ; i++) {	bag.add(Fleming, SAndS); }
			for (int i = 0; i < 7 ; i++) {	bag.add(Fleming, BitOfStuff); }
			for (int i = 0; i < 1 ; i++) {	bag.add(Fleming, EvilVillain); }
			
			
			System.out.println("isEmpty() is " + bag.isEmpty());
			System.out.println("size() is " + bag.size());
			System.out.println("uniqueKeySize() is " + bag.uniqueKeySize());
			System.out.println("uniqueKeyValuePairSize() is " + bag.uniqueKeyValuePairSize());
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println(e.toString());
			
		}
		
		try {	
			System.out.println("iterating bag using for-each gives");
			for (IMiniKeyValuePair<IMiniKeyValuePair<Author, CharacterType>, Integer> pair : bag) {
				System.out.println(pair);
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println(e.toString());
			e.printStackTrace();
			
		}
		
		try {	
			System.out.println("iterating bag using plain mini iterator");
			IMiniIterator<IMiniKeyValuePair<IMiniKeyValuePair<Author, CharacterType>,Integer>> it
								= bag.miniIterator();
			while(it.current() != null) {
				System.out.println("it.current() is " + it.current());
				it.next();
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println(e.toString());
			e.printStackTrace();
			
		}
		
		try {	
			System.out.println("iterating bag using bag iterator with current()");
			IMiniBagPairsIterator<Author, CharacterType> it = bag.bagIterator();
			while(it.current() != null) {
				System.out.println("it.current() is " + it.current());
				it.next();
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println(e.toString());
			e.printStackTrace();
			
		}
		
		try {	
			System.out.println("iterating bag using bag iterator with currentKey()," +
					" currentValue() and currentCount() we can output as");
			IMiniBagPairsIterator<Author, CharacterType> it = bag.bagIterator();
			while(it.current() != null) {
				System.out.println("Author " + it.currentKey() + " uses character "
						+ it.currentValue() + " " + it.currentCount() + " times");
				it.next();
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println(e.toString());
			e.printStackTrace();
			
		}
		
		System.out.println();
		try {
			
			IMiniBagPairsIterator<Author, CharacterType> it = bag.bagIterator();
			
			System.out.println("Get it = bag.bagIterator, current is " + it.current());
			System.out.println("call it.removeCurrent is ");
			it.removeCurrent();
			System.out.println("iterating bag using for-each gives");
			for (IMiniKeyValuePair<IMiniKeyValuePair<Author, CharacterType>, Integer> pair : bag) {
				System.out.println(pair);
			}
			System.out.println();
			System.out.println("using it again without calling it.next(), current is " + it.current());
			it.next();
			System.out.println("after calling it.next(), current is " + it.current());
			
			
			}
		catch (Exception e) {
			System.out.println(e.toString());
			e.printStackTrace();
		}
		System.out.println();
		
		try {	
			IMiniBagPairsIterator<Author, CharacterType> it = bag.bagIterator();
			
			System.out.println("Get it = bag.bagIterator, current is " + it.current());
			System.out.println("call bag.removeOnePair("+it.currentKey()+", "+it.currentValue()+")");
			boolean isRemoved = bag.removeOnePair(it.currentKey(), it.currentValue());
			System.out.println("result is " + isRemoved);
			System.out.println("check call bag.containsPair("+it.currentKey()+", "+it.currentValue()+")");
			boolean contains = bag.containsPair(it.currentKey(), it.currentValue());
			System.out.println("result is " + contains);
			System.out.println();
			System.out.println("Further operations using it without calling it = bagIterator() again");
			System.out.println("could result in concurrent modification exceptions:");
			System.out.println("but whether these happen depend on is not easily predicted");
			System.out.println(" - depends on the interaction of Java iterators and my iterators");
			try {
				System.out.println("this indicates a problem because ");
				System.out.print("using it again without calling it.next(), it.current() now is ");
				System.out.println(it.current());
				it = bag.bagIterator();
				System.out.print("new it = bag.bagIterator(), it.current() now is ");
				System.out.println(it.current());
			}
			catch (Exception e) {
				System.out.println(e.toString());
			}
			try {
				System.out.print("try calling it.next() instead:");
				it.next();
				System.out.print("it.current() now is ");
				System.out.println(it.current());
			}
			catch (Exception e) {
				System.out.println(e.toString());
			}
			
			
		}
		catch (Exception e) {
			System.out.println(e.toString());
		}
		System.out.println();
		
		
		try {	
			IMiniBagPairsIterator<Author, CharacterType> it = bag.bagIterator();
			it.next();
			Author nextAuthor = it.currentKey();
			CharacterType nextCT = it.currentValue();
			System.out.println("iterating bag using for-each gives");
			for (IMiniKeyValuePair<IMiniKeyValuePair<Author, CharacterType>, Integer> pair : bag) {
				System.out.println(pair);
			}
			
			it = bag.bagIterator();
			
			System.out.println("Get it = bag.bagIterator, current is " + it.current());
			System.out.println("remove the next pair in the list, call bag.removeAllPair("+nextAuthor+", "+nextCT+")");
			boolean isRemoved = bag.removeAllPair(nextAuthor, nextCT);
			System.out.println("result is " + isRemoved);
			System.out.println("check call bag.containsPair("+nextAuthor+", "+nextCT+")");
			boolean contains = bag.containsPair(nextAuthor, nextCT);
			System.out.println("result is " + contains);
			System.out.println();
			System.out.println("Further operations using it without calling it = bagIterator() again");
			System.out.println("could result in concurrent modification exceptions:");
			System.out.println("but whether these happen depend on is not easily predicted");
			System.out.println(" - depends on the interaction of Java iterators and my iterators");
			
			try {
				System.out.println("try iterating with while(it.current != null) ");
			
				while(it.current() != null) {
					System.out.println(it.current());
					it.next();
				}
				
			}
			catch (Exception e) {
				System.out.println(e.toString());
	
			}
			try {
				System.out.print("try calling it.next() instead:");
				it.next();
				System.out.print("it.current() now is ");
				System.out.println(it.current());
			}
			catch (Exception e) {
				System.out.println(e.toString());
			}
			
			
		}
		catch (Exception e) {
			System.out.println(e.toString());
		}
		System.out.println();
				
		try {
			
			IMiniBagPairsIterator<Author, CharacterType> it = 
				bag.bagIterator();
			System.out.println("try to use iteration on MiniBagIterator " +
					"to remove everything in bag,");
			System.out.println("but forgetting to call next() after removeCurrent()");
			System.out.println("(This should result in only one removal)");
			while(it.current() != null) {
				System.out.println("removing " + it.current());
				it.removeCurrent();
				//it.next();
			}
			System.out.println("size now is " + bag.size() + ", and isEmpty is " + bag.isEmpty());
						
		}
		catch (Exception e) {
			System.out.println(e.toString());
			
		}
		System.out.println();
		try {
			
			IMiniBagPairsIterator<Author, CharacterType> it = 
				bag.bagIterator();
			System.out.println("try to use iteration on MiniBagIterator " +
					"to remove everything in anotherBag, doing it properly");
			while(it.current() != null) {
				System.out.println("removing " + it.current());
				it.removeCurrent();
				it.next();
			}
			System.out.println("size now is " + bag.size() + ", and isEmpty is " + bag.isEmpty());
			
		}
		catch (Exception e) {
			System.out.println(e.toString());
			e.printStackTrace();
		}
		System.out.println();
		
		
	}
	
	private static class Author implements Comparable<Author>{
		String name;
		private Author() {}
		public Author(String name) {this.name = name; }
		public String getName() { return name; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			Author that = (Author) o;
			return this.name.equals(that.getName());
		}
		public int hashCode() {
			return name.hashCode();
		}
		public String toString() {return name; }
		public int compareTo(Author that) {
			//int retvalue = 0;
			//if (this.name.length() < that.getName().length()) retvalue = -1;
			//if (this.name.length() < that.getName().length()) retvalue = 1;
			return (this.name.compareTo(that.getName()));
		}
	}
	
	private static class CharacterType {
		String description;
		private CharacterType() {}
		public CharacterType(String description) {this.description = description; }
		public String getDescription() { return description; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			CharacterType that = (CharacterType) o;
			return this.description.equals(that.getDescription());
		}
		public int hashCode() {
			return description.hashCode();
		}
		public String toString() {return description; }
		public int compareTo(CharacterType that) {
			return (this.description.compareTo(that.getDescription()));
		}
	}

}
