package testing;

import minicollection.*;

/**
 * @author Jenny Harlow
 * 
 * Test bags of singles
 *
 */
public class TestBasicBagSingles {
	
	public static void main(String[] args) {
		
		IFactoryMiniBagSingles<String> factory = new FactoryMiniBagSingles<String>();
		
		IMiniBagSingles<String> bag = factory.makeBagSingles();
		System.out.println("made empty bag");
		
		System.out.println("checking configuration");
		System.out.println("getMembershipMode() = " + bag.getMembershipMode());
		System.out.println();
		
		
		System.out.println("IsEmpty is " + bag.isEmpty());
		System.out.println("Size is " + bag.size());
		
		bag.add("Jenny");
		bag.add("Wal");
		bag.add("Linda");
		
		System.out.println("added Jenny, Wal, Linda");
		System.out.println("IsEmpty is " + bag.isEmpty());
		System.out.println("Size is " + bag.size());
		
		try {
			System.out.println("iterating using Iterator");
			IMiniIterator<IMiniKeyValuePair<String,Integer>> it;
			it = bag.miniIterator();
			while(it.current() != null) {
				System.out.print(it.current() + "  ");
				it.next();
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		
		try {
			System.out.println("adding duplicates Wals and Jennys");
			bag.add("Wal");
			bag.add("Wal");
			bag.add("Jenny");
			
			System.out.println("Size is " + bag.size());
			System.out.println("uniqueKeySize is " + bag.uniqueKeySize());
					
			System.out.println("iterating with for each");
			for (IMiniKeyValuePair<String,Integer> pair : bag) {
				System.out.print(pair + " ");}
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		
		try {
			IMiniBagSinglesIterator<String> it = bag.bagIterator();
			String s  = it.currentKey();
			while (it.current() != null && it.currentCount() < 2) {
				it.next();
				s = it.currentKey();
			}
			System.out.println("check if bag contains " + s);
			System.out.println("bag.contains(" + s + ") = " + bag.contains(s));
			int count = bag.count(s);
			System.out.println("count for " + s + " is " + bag.count(s));
			System.out.println("using bag.remove(" + s + "):");
			while (count > 0) {
				bag.removeOne(s);
				count = bag.count(s);
				System.out.println("removed one, count for " + s + " is " + bag.count(s));
			}
			System.out.println("bag.contains(" + s + ") = " + bag.contains(s));
			
			try {
				System.out.println("try using bag.remove(" + s + ") again:");
				bag.removeOne(s);
				System.out.println("count for " + s + " is " + bag.count(s));
			}
			catch (Exception e) {
				System.out.println("That resulted in an exception:");
				System.out.println(e.toString());
			}
			System.out.println();
			
			System.out.println("add " + s + " back in ");
			bag.add(s);
			System.out.println("bag.contains(" + s + ") = " + bag.contains(s));
			count = bag.count(s);
			System.out.println("count for " + s + " is " + bag.count(s));
			
			
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
			
		try {
			System.out.println("iterating using the BagSinglesIterator, in pairs form");
			IMiniBagSinglesIterator<String> it = bag.bagIterator();
			while(it.current() != null) {
				System.out.print(it.current() + "  ");
				it.next();
			}
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		
		try {
			System.out.println("iterating using the BagSinglesIterator, " +
					"using currentKey() and currentCount()");
			IMiniBagSinglesIterator<String> it = bag.bagIterator();
			while(it.current() != null) {
				System.out.println("Key " + it.currentKey() + " has count " + it.currentCount());
				it.next();
			}
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		
		try {
			System.out.println("getting out the sequence of keys and counts");
			IMiniSequence<IMiniKeyValuePair<String, Integer>> keysAndCounts =
				bag.getKeysAndCounts();
			System.out.println("size is " + keysAndCounts.size() + " and pairs are:");
			for (IMiniKeyValuePair<String, Integer> pair : keysAndCounts) {
				System.out.println(pair);
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
		
	}

}
