package testing;

import java.util.*;

import minicollection.*;

/**
 * @author Jenny Harlow
 * 
 * Test sequences
 *
 */
public class TestBasicSequence {
	
	public static void main(String[] args) {
		IFactoryMiniSequence<String> factory = new FactoryMiniSequence<String>();
		
		IMiniSequence<String> seq = factory.makeSequence();
		
		System.out.println("made empty sequence, checking configuration");
		System.out.println("getMembershipMode() = " + seq.getMembershipMode());
		System.out.println("getOrderingMode() = " + seq.getOrderingMode());
		System.out.println("getAccessMode() = " + seq.getAccessMode());
		System.out.println();
		
		
		System.out.println("IsEmpty is " + seq.isEmpty());
		System.out.println("Size is " + seq.size());
		seq.add("Wal");
		seq.add("Richard");
		seq.add("Jenny");
		seq.add("Wal");
		seq.add("Linda");
		seq.add("Wal");
		
		
		System.out.println("added Wal, Richard, Jenny, Wal, Linda, Wal");
		System.out.println("IsEmpty is " + seq.isEmpty());
		System.out.println("Size is " + seq.size());
		
		System.out.println("iterating");
		IMiniSequenceIterator<String> it;
		it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.next();
		}
		System.out.println();
		
		System.out.println("iterating from the end");
		it = seq.end();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.previous();
		}
		System.out.println();
		
		IFactoryMiniSequence<Integer> intfactory = new FactoryMiniSequence<Integer>();
		
		IMiniSequence<Integer> indexes = intfactory.makeSequence();
		
		String find = "Wal";
		System.out.println("finding the positions of all the " + find + "s");
		
		int index = 0;
		while (index != -1 && index < seq.size()) {
			index = seq.nextIndexOf(index, find);
			System.out.println("found " + seq.getAtIndex(index) + " at index " + index);
			indexes.add(index);
			index++;
		}
		
		System.out.println("use removeAll to remove all the " + find + "s");
		seq.removeAll(find);
		boolean contains = seq.contains(find);
		System.out.println("after removeAll, seq.contains(" + find + ") = " + contains);
		System.out.println("use add to add " + find + " back at the same indices");
		for (int ind : indexes) {
			seq.add(find);
		}
		System.out.println("iterating");
		it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.next();
		}
		System.out.println();
		
		find = "Richard";
		IMiniSequenceIterator<String> sit = seq.seqIterator(seq.firstIndexOf(find));
		System.out.println("finding Richard with seqIterator(seq.firstIndexOf(find), " +
				sit.current());
		System.out.println("used same iterator to delete Richard: sequence now is");
		sit.previous();
		sit.removeCurrent();
		it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.next();
		}
		System.out.println();
		
		System.out.println("Trying Iterator and for each");
		for(String s : seq) {System.out.print(s + "  ");}
		System.out.println();
		
		System.out.println("removing duplicates: sequence now is:");
		seq.removeDuplicates();
		for(String s : seq) {System.out.print(s + "  ");}
		System.out.println();
		
		
		seq.sort(new CustomComparator());
		System.out.println("sorted by length, sequence now is");
		for(String s : seq) {System.out.print(s + "  ");}
		System.out.println();
	}

	private static class CustomComparator
	implements Comparator<String>
	{
		public int compare(String s1, String s2) {
			int retvalue = 0;
			if (s1.length() < s2.length()) retvalue = -1;
			if (s1.length() > s2.length()) retvalue = 1;
			return retvalue;
		}
	}
}
