package testing;


import java.util.*;

import minicollection.*;


/**
 * @author Jenny Harlow
 * 
 * Test custom-sorted sequence decorator
 *
 */
public class TestCustomSortedSequence {


	public static void main(String[] args) {
		
		IFactoryMiniSequence<Uncomparable> factory = 
			new FactoryMiniSequence<Uncomparable>();
		
		IMiniSequence<Uncomparable> seq = factory.makeSequence();
		
		seq.add(new Uncomparable("Jenny"));
		seq.add(new Uncomparable("Wal"));
		seq.add(new Uncomparable("Linda"));
		seq.add(new Uncomparable("Humperdink"));
		seq.add(new Uncomparable("Gardinia"));
		
		System.out.println("sequence is");
		IMiniSequenceIterator<Uncomparable> it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + " ");
			it.next();
		}
		System.out.println();
		
		IFactoryMiniSequence<Uncomparable> cs_factory = 
			new FactoryMiniSequenceCustomSorted<Uncomparable>(new CustomComparator());
		
		IMiniSequence<Uncomparable> sortedSeq = cs_factory.makeSequence(seq);
		
		System.out.println("made sorted sequence, checking configuration");
		System.out.println("getMembershipMode() = " + sortedSeq.getMembershipMode());
		System.out.println("getOrderingMode() = " + sortedSeq.getOrderingMode());
		System.out.println("getAccessMode() = " + sortedSeq.getAccessMode());
		System.out.println();
		
		
		System.out.println("with custom sorting by length, sequence now is");
		for(Uncomparable s : sortedSeq) {System.out.print(s + "  ");}
		System.out.println();
		
		System.out.println("finding the positions of all the Wals");
		Uncomparable anotherWal = new Uncomparable("Wal");
		int index = sortedSeq.firstIndexOf(anotherWal);
		while (index != -1) {
			System.out.println("found " + sortedSeq.getAtIndex(index) + " at index " + index);
			index = sortedSeq.nextIndexOf(index+1, anotherWal);
		}
		
		sortedSeq.add(new Uncomparable("Jo"));
		sortedSeq.add(new Uncomparable("Gareth"));
		System.out.println("added Jo and Gareth, sequence now is");
		for(Uncomparable s : sortedSeq) {System.out.print(s + "  ");}
		System.out.println();
		
		String s = "Marmalade";
		try {
			
			System.out.println("Trying to add " + s);
			sortedSeq.add(new Uncomparable(s));
			for(Uncomparable ss : sortedSeq) {System.out.print(ss + "  ");}
			System.out.println();
			
		}
		catch (UnsupportedOperationException e) {
            //do something clever with the exception
            System.out.println(e.toString());
		}
		
		try {
						
			System.out.println("Trying to add at " + s + "index 0");
			sortedSeq.add(new Uncomparable(s));
			for(Uncomparable ss : sortedSeq) {System.out.print(ss + "  ");}
			System.out.println();
		}
		catch (UnsupportedOperationException e) {
            //do something clever with the exception
            System.out.println(e.toString());
		}
		
			
		try {
			
			System.out.println("Trying to sort with comparator");
			sortedSeq.sort(new CustomComparator());
			
			}
		catch (Exception e) {
            //do something clever with the exception
            System.out.println(e.toString());
		}
		
		try {
			
			System.out.println("Trying to sort the list using natural ordering");
			System.out.println("This should throw an exception");
			sortedSeq.sort();
			
			System.out.println("Failed to throw exception on sort");
		}
		catch (Exception e) {
            //do something clever with the exception
            System.out.println(e.toString());
		}
		
				
		
		System.out.println("removing duplicates: sequence now is:");
		sortedSeq.removeDuplicates();
		for(Uncomparable ss : sortedSeq) {System.out.print(ss + "  ");}
		System.out.println();
		
		
		System.out.println("iterating from the start");
		it = sortedSeq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.next();
		}
		System.out.println();
		
		System.out.println("iterating from the end");
		it = sortedSeq.end();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.previous();
		}
		System.out.println();
	}

	private static class Uncomparable {
		String name;
		private Uncomparable() {}
		public Uncomparable(String name) {this.name = name; }
		public String getName() { return name; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			Uncomparable that = (Uncomparable) o;
			return this.name.equals(that.getName());
		}
		public int hashCode() {
			return name.hashCode();
		}
		public String toString() {return name; }
	}
	
	
	private static class CustomComparator
	implements Comparator<Uncomparable>
	{
		public int compare(Uncomparable s1, Uncomparable s2) {
			int retvalue = 0;
			if (s1.getName().length() < s2.getName().length()) retvalue = -1;
			if (s1.getName().length() > s2.getName().length()) retvalue = 1;
			return retvalue;
		}
	}

}
