package testing;


import java.util.*;

import rejects.MiniSequenceSelfSortedDecorator;

import minicollection.*;


/**
 * @author Jenny Harlow
 * 
 * Test trying to create a NaturallySortedSequence
 * with a type that does not implement Comparable.
 * 
 * This code should not compile.
 *
 */
public class TestImpossibleNaturallySortedSequence {

	

	public static void main(String[] args) {
		IFactoryMiniSequence<Uncomparable> factory = 
			new FactoryMiniSequence<Uncomparable>();
		
		IMiniSequence<Uncomparable> seq = factory.makeSequence();
		
		seq.add(new Uncomparable("Jenny"));
		seq.add(new Uncomparable("Wal"));
		seq.add(new Uncomparable("Linda"));
		seq.add(new Uncomparable("Humperdink"));
		seq.add(new Uncomparable("Gardinia"));
		
		System.out.println("adding duplicates Wals, at specific indices, and Richard before Jenny");
		seq.addAtIndex(0, new Uncomparable("Wal"));
		seq.addAtIndex(seq.size(), new Uncomparable("Wal"));
		int jindex = seq.firstIndexOf(new Uncomparable("Jenny"));
		seq.addAtIndex(jindex, new Uncomparable("Richard"));
				
		System.out.println("sequence is");
		IMiniSequenceIterator<Uncomparable> it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + " ");
			it.next();
		}
		System.out.println();
		
		//this should not compile
		IFactoryMiniSequence<Uncomparable> ns_factory = 
			new FactoryMiniSequenceNaturallySorted<Uncomparable>();
		
		IMiniSequence<Uncomparable> sortedSeq = ns_factory.makeSequence(seq);
		
	}

	private static class Uncomparable {
		String name;
		private Uncomparable() {}
		public Uncomparable(String name) {this.name = name; }
		public String getName() {return name; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			Uncomparable that = (Uncomparable) o;
			return this.name.equals(that.getName());
		}
		public int hashCode() {
			return name.hashCode();
		}
		public String toString() {return name; }
		/*
		public int compareTo(NaturallyComparable that) {
			int retvalue = 0;
			if (this.name.length() < that.getName().length()) retvalue = -1;
			if (this.name.length() < that.getName().length()) retvalue = 1;
			return retvalue;
		}
		*/
	}
	
	
	private static class CustomComparator
	implements Comparator<Uncomparable>
	{
		public int compare(Uncomparable s1, Uncomparable s2) {
			int retvalue = 0;
			if (s1.getName().length() < s2.getName().length()) retvalue = -1;
			if (s1.getName().length() > s2.getName().length()) retvalue = 1;
			return retvalue;
		}
	}

}
