package testing;


import java.util.*;

import minicollection.*;


/**
 * @author Jenny Harlow
 * 
 * Test flexi-add sequence
 *
 */
public class TestNaturallyComparableFlexiAddSequence {


	public static void main(String[] args) {
		
		IFactoryMiniSequenceFlexiAdd<NaturallyComparable> factory = 
			new FactoryMiniSequenceFlexiAdd<NaturallyComparable>();
		
		IMiniSequenceFlexiAdd<NaturallyComparable> seq = factory.makeSequence();
		
		seq.add(new NaturallyComparable("Jenny"));
		seq.add(new NaturallyComparable("Wal"));
		seq.add(new NaturallyComparable("Linda"));
		seq.add(new NaturallyComparable("Humperdink"));
		seq.add(new NaturallyComparable("Gardinia"));
		
		System.out.println("adding duplicates Wals, at specific indices, and Richard before Jenny");
		seq.addAtIndex(0, new NaturallyComparable("Wal"));
		seq.addAtIndex(seq.size(), new NaturallyComparable("Wal"));
		int jindex = seq.firstIndexOf(new NaturallyComparable("Jenny"));
		seq.addAtIndex(jindex, new NaturallyComparable("Richard"));
				
		System.out.println("sequence is");
		IMiniSequenceIterator<NaturallyComparable> it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + " ");
			it.next();
		}
		System.out.println();
		
		
		System.out.println("made sorted sequence, checking configuration");
		System.out.println("getMembershipMode() = " + seq.getMembershipMode());
		System.out.println("getOrderingMode() = " + seq.getOrderingMode());
		System.out.println("getAccessMode() = " + seq.getAccessMode());
		System.out.println();
		
		
		System.out.println("with natural name sorting, sequence now is");
		for(NaturallyComparable s : seq) {System.out.print(s + "  ");}
		System.out.println();
		
		System.out.println("finding the positions of all the Wals");
		NaturallyComparable anotherWal = new NaturallyComparable("Wal");
		//int index = seq.firstIndexOf(anotherWal);
		int index = 0;
		while (index != -1 && index < seq.size()) {
			index = seq.nextIndexOf(index, anotherWal);
			System.out.println("found " + seq.getAtIndex(index) + " at index " + index);
			index++;
		}
		
		seq.add(new NaturallyComparable("Jo"));
		seq.add(new NaturallyComparable("Gareth"));
		System.out.println("added Jo and Gareth, sequence now is");
		for(NaturallyComparable s : seq) {System.out.print(s + "  ");}
		System.out.println();
		
		String s = "Marmalade";
		try {
			
			System.out.println("Trying to add " + s);
			seq.add(new NaturallyComparable(s));
			for(NaturallyComparable ss : seq) {System.out.print(ss + "  ");}
			System.out.println();
			
		}
		catch (Exception e) {
            //do something clever with the exception
            System.out.println(e.toString());
		}
		
		try {
						
			System.out.println("Trying to add at " + s + "index 0");
			seq.add(new NaturallyComparable(s));
			for(NaturallyComparable ss : seq) {System.out.print(ss + "  ");}
			System.out.println();
		}
		catch (Exception e) {
            //do something clever with the exception
            System.out.println(e.toString());
		}
		
		try {
			s = "Wibble";
			int windex = 1;
			System.out.println("Trying to reset element " + seq.getAtIndex(windex) + " at index " + windex + " to " + s);
			seq.resetAtIndex(1, new NaturallyComparable(s));
			for(NaturallyComparable ss : seq) {System.out.print(ss + "  ");}
			System.out.println();
		}
		catch (Exception e) {
            //do something clever with the exception
            System.out.println(e.toString());
		}
		
		
		try {
			
			System.out.println("Trying to sort with comparator");
			seq.sort(new CustomComparator());
			
			for(NaturallyComparable ss : seq) {System.out.print(ss + "  ");}
			System.out.println();
			
		}
		catch (Exception e) {
            //do something clever with the exception
            System.out.println(e.toString());
		}
		

		try {
			
			System.out.println("Trying to sort the list with natural ordering");
			seq.sort();
			System.out.println("list now is");
			for(NaturallyComparable ss : seq) {System.out.print(ss + "  ");}
			System.out.println();
			
		}
		catch (Exception e) {
            //do something clever with the exception
            System.out.println(e.toString());
		}
		
				
		
		System.out.println("removing duplicates: sequence now is:");
		seq.removeDuplicates();
		for(NaturallyComparable ss : seq) {System.out.print(ss + "  ");}
		System.out.println();
		
		
		System.out.println("iterating from the start");
		it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.next();
		}
		System.out.println();
		
		System.out.println("iterating from the end");
		it = seq.end();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.previous();
		}
		System.out.println();
	}

	
	private static class NaturallyComparable implements Comparable<NaturallyComparable>{
		String name;
		private NaturallyComparable() {}
		public NaturallyComparable(String name) {this.name = name; }
		public String getName() {return name; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			NaturallyComparable that = (NaturallyComparable) o;
			return this.name.equals(that.getName());
		}
		public int hashCode() {
			return name.hashCode();
		}
		public String toString() {return name; }
		public int compareTo(NaturallyComparable that) {
			//int retvalue = 0;
			//if (this.name.length() < that.getName().length()) retvalue = -1;
			//if (this.name.length() < that.getName().length()) retvalue = 1;
			return (this.name.compareTo(that.getName()));
		}
	}
	
	
	private static class CustomComparator
	implements Comparator<NaturallyComparable>
	{
		public int compare(NaturallyComparable s1, NaturallyComparable s2) {
			int retvalue = 0;
			if (s1.getName().length() < s2.getName().length()) retvalue = 1;
			if (s1.getName().length() > s2.getName().length()) retvalue = -1;
			return retvalue;
		}
	}
	
}
