package testing;

import java.util.*;

import minicollection.*;

/**
 * @author Jenny Harlow
 * 
 * Test sequence iterator
 *
 */
public class TestSequenceIterator {
	

	public static void main(String[] args) {
		
		IFactoryMiniSequence<String> factory = new FactoryMiniSequence<String>();
		
		IMiniSequence<String> seq = factory.makeSequence();
		
		System.out.println("made empty sequence, checking configuration");
		System.out.println("getMembershipMode() = " + seq.getMembershipMode());
		System.out.println("getOrderingMode() = " + seq.getOrderingMode());
		System.out.println("getAccessMode() = " + seq.getAccessMode());
		System.out.println();
		
		
		System.out.println("IsEmpty is " + seq.isEmpty());
		System.out.println("Size is " + seq.size());
		seq.add("Jenny");
		seq.add("Wal");
		seq.add("Linda");
		
		System.out.println("added Jenny, Wal, Linda");
		System.out.println("IsEmpty is " + seq.isEmpty());
		System.out.println("Size is " + seq.size());
		
		
		try {
			System.out.println("for each");
			for (String ss : seq )
				System.out.print(ss + "  ");
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.getMessage());
		}
		System.out.println();
		try {
			System.out.println("iterating using the MiniIterator");
			IMiniIterator<String> it;
			it = seq.miniIterator();
			while(it.current() != null) {
				System.out.print(it.current() + "  ");
				it.next();
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.getMessage());
		}
		System.out.println();
		try {
			System.out.println("iterating from the start using the SequenceIterator");
			IMiniSequenceIterator<String> it = seq.begin();
			while(it.current() != null) {
				System.out.print(it.current() + "  ");
				it.next();
			}
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		try {
			System.out.println("iterating from the end");
			IMiniSequenceIterator<String> it  = seq.end();
			while(it.current() != null) {
				System.out.print(it.current() + "  ");
				it.previous();
			}
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		try {
			int index = 0;
			System.out.println("it  = seq.seqIterator(" + index + ")");
			IMiniSequenceIterator<String> it  = seq.seqIterator(index);
			System.out.println("gives it.current() = ");
			System.out.println(it.current());
		}
		catch (IndexOutOfBoundsException e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		try {
			int index = 1;
			System.out.println("it  = seq.seqIterator(" + index + ")");
			IMiniSequenceIterator<String> it  = seq.seqIterator(index);
			System.out.println("gives it.current() = ");
			System.out.println(it.current());
		}
		catch (IndexOutOfBoundsException e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		try {	
			int index = 2;
			System.out.println("it  = seq.seqIterator(" + index + ")");
			IMiniSequenceIterator<String> it  = seq.seqIterator(index);
			System.out.println("gives it.current() = ");
			System.out.println(it.current());
		}
		catch (IndexOutOfBoundsException e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		try {	
			int index = 3;
			System.out.println("it  = seq.seqIterator(" + index + ")");
			IMiniSequenceIterator<String> it  = seq.seqIterator(index);
			System.out.println("gives it.current() = ");
			System.out.println(it.current());
		}
		catch (IndexOutOfBoundsException e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
		System.out.println();
		try {	
			int index = 4;
			System.out.println("it  = seq.seqIterator(" + index + ")");
			IMiniSequenceIterator<String> it  = seq.seqIterator(index);
			System.out.println("gives it.current() = ");
			System.out.println(it.current());
		}
		catch (IndexOutOfBoundsException e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
		System.out.println();
		try {	
			IMiniSequenceIterator<String> it  = seq.begin();
			String currentElement = it.current();
			System.out.println("it  = seq.begin(), current is " + currentElement);
			
			System.out.println("now trying removeCurrent: ");
			it.removeCurrent();
			System.out.println("it.current() == null is " 
					+ (it.current() == null));
			it.next();
			System.out.println("after calling it.next(), it.current() = " 
					+ it.current());
			seq.add(currentElement);
			System.out.println("put " + currentElement + " back in, sequence now is");
			for (String ss : seq) { System.out.print(ss + "  ");}
			System.out.println();
			System.out.println("set it to seq.end()");
			it  = seq.end();
			currentElement = it.current();
			System.out.println("current now is " + currentElement);
			System.out.println("now trying removeCurrent: ");
			it.removeCurrent();
			System.out.println("it.current() == null is " 
					+ (it.current() == null));
			it.next();
			System.out.println("after calling it.next(), it.current() = " 
					+ it.current());
			
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
		System.out.println();
		try {	
			String addString = "Gardinia";
			seq.add(addString);
			System.out.println("put " + addString + " in, sequence now is");
			for (String ss : seq) { System.out.print(ss + "  ");}
			System.out.println();
			
			System.out.println("try to use forward iteration to remove everything");
			IMiniSequenceIterator<String> it = seq.begin();
			while(it.current() != null) {
				System.out.println("removing " + it.current());
				it.removeCurrent();
				it.next();
			}
			System.out.println("size now is " + seq.size());
			System.out.println();
			
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
		System.out.println();
		try {	
			seq.add("Jenny");
			seq.add("Wal");
			seq.add("Linda");
			System.out.println("put three more in, sequence now is");
			for (String ss : seq) { System.out.print(ss + "  ");}
			System.out.println();
		
			System.out.println("try to use backward iteration to remove everything");
			IMiniSequenceIterator<String> it = seq.end();
			while(it.current() != null) {
				System.out.println("removing " + it.current());
				it.removeCurrent();
				it.previous();
			}
			System.out.println("size now is " + seq.size());
			System.out.println();
			
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
		
		try {	
			seq.add("Jenny");
			seq.add("Wal");
			seq.add("Linda");
			System.out.println("put three more in, sequence now is");
			for (String ss : seq) { System.out.print(ss + "  ");}
			System.out.println();
			
			Iterator<String> it = seq.iterator();
			System.out.println("got a Java iterator it = seq.iterator()");
			it.next();
			System.out.println("try to use it.remove() - this should throw an exception");
			it.remove();
				
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
			
		}
	}

	private static class CustomComparator
	implements Comparator<String>
	{
		public int compare(String s1, String s2) {
			int retvalue = 0;
			if (s1.length() < s2.length()) retvalue = -1;
			if (s1.length() > s2.length()) retvalue = 1;
			return retvalue;
		}
	}

}
