package testing;


import java.util.*;

import minicollection.*;


/**
 * @author Jenny Harlow
 * 
 * Test changing the implementation for a sequence
 *
 */
public class TestSortedSequenceChangeImplementation {

	
	public static void main(String[] args) {
		
		IMiniSequence<NaturallyComparable> seq = null;
		try {
		
			IFactoryMiniSequence<NaturallyComparable> factory = 
				new FactoryMiniSequence<NaturallyComparable>();
			
			seq = factory.makeSequence();
			
			seq.add(new NaturallyComparable("Jenny"));
			seq.add(new NaturallyComparable("Wal"));
			seq.add(new NaturallyComparable("Linda"));
			seq.add(new NaturallyComparable("Humperdink"));
			seq.add(new NaturallyComparable("Gardinia"));
			
			System.out.println("sequence is");
			IMiniSequenceIterator<NaturallyComparable> it = seq.begin();
			while(it.current() != null) {
				System.out.print(it.current() + "  ");
				it.next();
			}
			System.out.println();
		
		
			AccessMode accessMode = AccessMode.SEQUENTIAL;
			System.out.println("resetting the access mode for sequence to " + accessMode);
			seq.resetAccessMode(accessMode);
			System.out.println("sequence now is:");
					
			for(NaturallyComparable s : seq) {System.out.print(s + "  ");}
			System.out.println();
			System.out.println();
			
			NaturallyComparable Aardvark = new NaturallyComparable("Aardvark");
			seq.add(Aardvark);
			System.out.println("adding new element " + Aardvark);
			System.out.println("sequence now is:");
					
			for(NaturallyComparable s : seq) {System.out.print(s + "  ");}
			System.out.println();
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println(e.toString());
		}
		
		try {
			IFactoryMiniSequence<NaturallyComparable> ns_factory = 
				new FactoryMiniSequenceNaturallySorted<NaturallyComparable>();
			
			IMiniSequence<NaturallyComparable> sortedSeq = ns_factory.makeSequence(seq);
			
			System.out.println("made sorted sequence, checking configuration");
			System.out.println("getMembershipMode() = " + sortedSeq.getMembershipMode());
			System.out.println("getOrderingMode() = " + sortedSeq.getOrderingMode());
			System.out.println("getAccessMode() = " + sortedSeq.getAccessMode());
			System.out.println();
			
			
			System.out.println("with natural name sorting, sequence now is");
			for(NaturallyComparable s : sortedSeq) {System.out.print(s + "  ");}
			System.out.println();
			System.out.println();
		
		
			AccessMode accessMode = AccessMode.SEQUENTIAL;
			System.out.println("resetting the access mode for the sorted sequence to " + accessMode);
			sortedSeq.resetAccessMode(accessMode);
			System.out.println("sequence now is:");
					
			for(NaturallyComparable s : sortedSeq) {System.out.print(s + "  ");}
			System.out.println();
			System.out.println();
			
			NaturallyComparable Aardvark = new NaturallyComparable("Aardvark");
			sortedSeq.add(Aardvark);
			System.out.println("adding new element " + Aardvark);
			System.out.println("sequence now is:");
					
			for(NaturallyComparable s : sortedSeq) {System.out.print(s + "  ");}
			System.out.println();
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println(e.toString());
		}
		
		try {
			IFactoryMiniSequence<NaturallyComparable> unique_factory = 
				new FactoryMiniSequenceUniqueOnly<NaturallyComparable>();
			
			IMiniSequence<NaturallyComparable> uniqueSeq = unique_factory.makeSequence(seq);
			
			System.out.println();
			System.out.println("made unique sequence, checking configuration");
			System.out.println("getMembershipMode() = " + uniqueSeq.getMembershipMode());
			System.out.println("getOrderingMode() = " + uniqueSeq.getOrderingMode());
			System.out.println("getAccessMode() = " + uniqueSeq.getAccessMode());
			System.out.println();
			
			System.out.println("enforcing uniqueness, sequence now is");
			for(NaturallyComparable s : uniqueSeq) {System.out.print(s + "  ");}
			System.out.println();
			System.out.println();
		
		
			AccessMode accessMode = AccessMode.SEQUENTIAL;
			System.out.println("resetting the access mode for the uniques only sequence to " + accessMode);
			uniqueSeq.resetAccessMode(accessMode);
			System.out.println("sequence now is:");
					
			for(NaturallyComparable s : uniqueSeq) {System.out.print(s + "  ");}
			System.out.println();
			System.out.println();
			
			NaturallyComparable Wal = new NaturallyComparable("Wal");
			uniqueSeq.add(Wal);
			System.out.println("adding new element " + Wal);
			System.out.println("sequence now is:");
					
			for(NaturallyComparable s : uniqueSeq) {System.out.print(s + "  ");}
			System.out.println();
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println(e.toString());
		}
		
	}
	
	
	private static class NaturallyComparable implements Comparable<NaturallyComparable>{
		String name;
		private NaturallyComparable() {}
		public NaturallyComparable(String name) {this.name = name; }
		public String getName() {return name; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			NaturallyComparable that = (NaturallyComparable) o;
			return this.name.equals(that.getName());
		}
		public int hashCode() {
			return name.hashCode();
		}
		public String toString() {return name; }
		public int compareTo(NaturallyComparable that) {
			return (this.name.compareTo(that.getName()));
		}
	}
	
	
	private static class CustomComparator
	implements Comparator<NaturallyComparable>
	{
		public int compare(NaturallyComparable s1, NaturallyComparable s2) {
			int retvalue = 0;
			if (s1.getName().length() < s2.getName().length()) retvalue = 1;
			if (s1.getName().length() > s2.getName().length()) retvalue = -1;
			return retvalue;
		}
	}

}

