package testing;


import java.util.*;

import minicollection.*;


/**
 * @author Jenny Harlow
 * 
 * Test sequential access implementation for a sequence
 *
 */
public class TestSortedSequenceSequentialAccess {

	
	
	public static void main(String[] args) {
		
		IFactoryMiniSequence<NaturallyComparable> factory = 
			new FactoryMiniSequence<NaturallyComparable>();
		
		IMiniSequence<NaturallyComparable> seq = factory.makeSequence();
		
		System.out.println("made basic sequence, no accesss specified - checking configuration");
		System.out.println("getMembershipMode() = " + seq.getMembershipMode());
		System.out.println("getOrderingMode() = " + seq.getOrderingMode());
		System.out.println("getAccessMode() = " + seq.getAccessMode());
		System.out.println();
		
		seq.add(new NaturallyComparable("Jenny"));
		seq.add(new NaturallyComparable("Wal"));
		seq.add(new NaturallyComparable("Linda"));
		seq.add(new NaturallyComparable("Humperdink"));
		seq.add(new NaturallyComparable("Gardinia"));
		
		System.out.println("sequence is");
		IMiniSequenceIterator<NaturallyComparable> it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + " ");
			it.next();
		}
		System.out.println();
		
		System.out.println("making sorted sequence, specifying sequential access - checking configuration");
		
		IFactoryMiniSequence<NaturallyComparable> ns_factory = 
			new FactoryMiniSequenceNaturallySorted<NaturallyComparable>();
		
		IMiniSequence<NaturallyComparable> sortedSeq = ns_factory.makeSequence(seq, AccessMode.SEQUENTIAL);
		
		System.out.println("made sorted sequence, checking configuration");
		System.out.println("getMembershipMode() = " + sortedSeq.getMembershipMode());
		System.out.println("getOrderingMode() = " + sortedSeq.getOrderingMode());
		System.out.println("getAccessMode() = " + sortedSeq.getAccessMode());
		System.out.println();
		
		
		System.out.println("making a basic sequence, sequential accesss specified - checking configuration");
		
		seq = factory.makeSequence(AccessMode.SEQUENTIAL);
		
		System.out.println("made basic sequence, no accesss specified - checking configuration");
		System.out.println("getMembershipMode() = " + seq.getMembershipMode());
		System.out.println("getOrderingMode() = " + seq.getOrderingMode());
		System.out.println("getAccessMode() = " + seq.getAccessMode());
		System.out.println();
		
		seq.add(new NaturallyComparable("Jenny"));
		seq.add(new NaturallyComparable("Wal"));
		seq.add(new NaturallyComparable("Linda"));
		seq.add(new NaturallyComparable("Humperdink"));
		seq.add(new NaturallyComparable("Gardinia"));
		
		System.out.println("sequence is");
		it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + " ");
			it.next();
		}
		System.out.println();
		
		System.out.println("making sorted sequence, specifying random access - checking configuration");
		
		sortedSeq = ns_factory.makeSequence(seq, AccessMode.RANDOM);
		
		System.out.println("made sorted sequence, checking configuration");
		System.out.println("getMembershipMode() = " + sortedSeq.getMembershipMode());
		System.out.println("getOrderingMode() = " + sortedSeq.getOrderingMode());
		System.out.println("getAccessMode() = " + sortedSeq.getAccessMode());
		System.out.println();
		
	}

	private static class NaturallyComparable implements Comparable<NaturallyComparable>{
		String name;
		private NaturallyComparable() {}
		public NaturallyComparable(String name) {this.name = name; }
		public String getName() {return name; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			NaturallyComparable that = (NaturallyComparable) o;
			return this.name.equals(that.getName());
		}
		public int hashCode() {
			return name.hashCode();
		}
		public String toString() {return name; }
		public int compareTo(NaturallyComparable that) {
			return (this.name.compareTo(that.getName()));
		}
	}
	
	
	private static class CustomComparator
	implements Comparator<NaturallyComparable>
	{
		public int compare(NaturallyComparable s1, NaturallyComparable s2) {
			int retvalue = 0;
			if (s1.getName().length() < s2.getName().length()) retvalue = 1;
			if (s1.getName().length() > s2.getName().length()) retvalue = -1;
			return retvalue;
		}
	}

}
