package testing;


import java.util.*;

import minicollection.*;


/**
 * @author Jenny Harlow
 * 
 * Test what happens to original sequence when you decorate it
 *
 */
public class TestSortedSequenceWhatHappensToOriginalSequences {

	public static void main(String[] args) {

		IFactoryMiniSequence<NaturallyComparable> factory = 
			new FactoryMiniSequence<NaturallyComparable>();
		
		IMiniSequence<NaturallyComparable> seq = factory.makeSequence();
		
		seq.add(new NaturallyComparable("Jenny"));
		seq.add(new NaturallyComparable("Wal"));
		seq.add(new NaturallyComparable("Linda"));
		seq.add(new NaturallyComparable("Humperdink"));
		seq.add(new NaturallyComparable("Gardinia"));
		
		System.out.println("sequence is");
		IMiniSequenceIterator<NaturallyComparable> it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + " ");
			it.next();
		}
		System.out.println();
		
		IFactoryMiniSequence<NaturallyComparable> ns_factory = 
			new FactoryMiniSequenceNaturallySorted<NaturallyComparable>();
		
		IMiniSequence<NaturallyComparable> sortedSeq = ns_factory.makeSequence(seq);
		
		System.out.println("made sorted sequence, checking configuration");
		System.out.println("getMembershipMode() = " + sortedSeq.getMembershipMode());
		System.out.println("getOrderingMode() = " + sortedSeq.getOrderingMode());
		System.out.println("getAccessMode() = " + sortedSeq.getAccessMode());
		System.out.println();
		
		
		System.out.println("with natural name sorting, sequence now is");
		for(NaturallyComparable s : sortedSeq) {System.out.print(s + "  ");}
		System.out.println();
		System.out.println();
		
		System.out.println("old sequence is");
		for(NaturallyComparable s : seq) {System.out.print(s + "  ");}
		System.out.println();
		System.out.println();
		
		/*
		System.out.println("finding the positions of all the Wals");
		NaturallyComparable anotherWal = new NaturallyComparable("Wal");
		//int index = sortedSeq.firstIndexOf(anotherWal);
		int index = 0;
		while (index != -1 && index < sortedSeq.size()) {
			index = sortedSeq.nextIndexOf(index, anotherWal);
			System.out.println("found " + sortedSeq.getAtIndex(index) + " at index " + index);
			index++;
		}
		
		sortedSeq.add(new NaturallyComparable("Jo"));
		sortedSeq.add(new NaturallyComparable("Gareth"));
		System.out.println("added Jo and Gareth, sequence now is");
		for(NaturallyComparable s : sortedSeq) {System.out.print(s + "  ");}
		System.out.println();
		
		String s = "Marmalade";
		try {
			
			System.out.println("Trying to add " + s);
			sortedSeq.add(new NaturallyComparable(s));
			for(NaturallyComparable ss : sortedSeq) {System.out.print(ss + "  ");}
			System.out.println();
			
		}
		catch (UnsupportedOperationException e) {
            //do something clever with the exception
            System.out.println(e.getMessage());
		}
		
		try {
						
			System.out.println("Trying to add at " + s + "index 0");
			sortedSeq.add(new NaturallyComparable(s));
			for(NaturallyComparable ss : sortedSeq) {System.out.print(ss + "  ");}
			System.out.println();
		}
		catch (UnsupportedOperationException e) {
            //do something clever with the exception
            System.out.println(e.getMessage());
		}
		
		try {
			s = "Wibble";
			int windex = 1;
			System.out.println("Trying to reset element " + sortedSeq.getAtIndex(windex) + " at index " + windex + " to " + s);
			sortedSeq.resetAtIndex(1, new NaturallyComparable(s));
			for(NaturallyComparable ss : sortedSeq) {System.out.print(ss + "  ");}
			System.out.println();
		}
		catch (UnsupportedOperationException e) {
            //do something clever with the exception
            System.out.println(e.getMessage());
		}
		
		
		try {
			
			System.out.println("Trying to sort with comparator");
			sortedSeq.sort(new CustomComparator());
			
			System.out.println("Failed to throw exception on sort with custom comparator");
		}
		catch (UnsupportedOperationException e) {
            //do something clever with the exception
            System.out.println(e.getMessage());
		}
		

		try {
			
			System.out.println("Trying to sort the list");
			sortedSeq.sort();
			
			System.out.println("Failed to throw exception on sort");
		}
		catch (UnsupportedOperationException e) {
            //do something clever with the exception
            System.out.println(e.getMessage());
		}
		
				
		
		System.out.println("removing duplicates: sequence now is:");
		sortedSeq.removeDuplicates();
		for(NaturallyComparable ss : sortedSeq) {System.out.print(ss + "  ");}
		System.out.println();
		
		
		System.out.println("iterating from the start");
		it = sortedSeq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.next();
		}
		System.out.println();
		
		System.out.println("iterating from the end");
		it = sortedSeq.end();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.previous();
		}
		System.out.println();
		*/
	}

	private static class NaturallyComparable implements Comparable<NaturallyComparable>{
		String name;
		private NaturallyComparable() {}
		public NaturallyComparable(String name) {this.name = name; }
		public String getName() {return name; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			NaturallyComparable that = (NaturallyComparable) o;
			return this.name.equals(that.getName());
		}
		public int hashCode() {
			return name.hashCode();
		}
		public String toString() {return name; }
		public int compareTo(NaturallyComparable that) {
			return (this.name.compareTo(that.getName()));
		}
	}
	
	
	private static class CustomComparator
	implements Comparator<NaturallyComparable>
	{
		public int compare(NaturallyComparable s1, NaturallyComparable s2) {
			int retvalue = 0;
			if (s1.getName().length() < s2.getName().length()) retvalue = 1;
			if (s1.getName().length() > s2.getName().length()) retvalue = -1;
			return retvalue;
		}
	}
	

}
