package testing;



import minicollection.*;

/**
 * @author Jenny Harlow
 * 
 * Test bags of unique pairs
 *
 */
public class TestUniqueBagPairs {

	
	public static void main(String[] args) {
		
		IMiniBagPairs<Author, CharacterType> uniquesBag = null;
		
		IFactoryMiniBagPairs<Author, CharacterType> factory = 
			new FactoryMiniBagPairsUniqueOnly<Author,CharacterType>();
		
		Author Heyer = new Author("Georgette Heyer");
		Author Simenon = new Author("Simenon");
		Author Pratchett = new Author("Terry Pratchett");
		Author Fleming = new Author("Ian Fleming");
		
		CharacterType StrongAndSilent = new CharacterType("Strong and Silent");
		CharacterType SAndS = new CharacterType("Strong and Silent");
		CharacterType IntelligentHeroine = new CharacterType("Intelligent Heroine");
		CharacterType FluffyBlonde = new CharacterType("Fluffy Blonde");
		CharacterType BitOfStuff = new CharacterType("Fluffy Blonde");
		CharacterType HeavyPoliceman = new CharacterType("Heavy Policeman");
		CharacterType DowntroddenWife = new CharacterType("Downtrodden Wife");
		CharacterType HenpeckedHusband = new CharacterType("Henpecked Husband");
		CharacterType EvilVillain = new CharacterType("Evil Villain");
		CharacterType MaleFriend = new CharacterType("Uncompetitive Male Friend");
		CharacterType Wizard = new CharacterType("Wizard");
		CharacterType Witch = new CharacterType("Witch");
		CharacterType Troll = new CharacterType("Troll");
		CharacterType Death = new CharacterType("Death");
		CharacterType Nemesis = new CharacterType("Death");
		CharacterType Retribution = new CharacterType("Death");
		
		try {
		
			uniquesBag = factory.makeBagPairs();
			
			System.out.println("Made a uniques only bag: checking size and configuration");
			System.out.println("getMembershipMode() = " + uniquesBag.getMembershipMode());
			System.out.println("IsEmpty() is " + uniquesBag.isEmpty());
			System.out.println("Size() is " + uniquesBag.size());
			System.out.println();
			
			for (int i = 0; i < 5 ; i++) {	uniquesBag.add(Heyer, StrongAndSilent); }
			
			for (int i = 0; i < 10 ; i++) {	uniquesBag.add(Pratchett, Wizard); }
			
			for (int i = 0; i < 7 ; i++) {	uniquesBag.add(Fleming, BitOfStuff); }
						
			System.out.println("Added 3 authors, 3 mappings");
			System.out.println("isEmpty() is " + uniquesBag.isEmpty());
			System.out.println("size() is " + uniquesBag.size());
			System.out.println("uniqueKeySize() is " + uniquesBag.uniqueKeySize());
			System.out.println("uniqueKeyValuePairSize() is " + uniquesBag.uniqueKeyValuePairSize());
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println(e.toString());
			
		}
		
		try {	
			System.out.println("iterating bag using bag iterator with currentKey(), currentValue() and currentCount() we can output as");
			IMiniBagPairsIterator<Author, CharacterType> it = uniquesBag.bagIterator();
			while(it.current() != null) {
				System.out.println("Author " + it.currentKey() + " uses character "
						+ it.currentValue() + " " + it.currentCount() + " times");
				it.next();
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println(e.toString());
			
		}
		
		
		try {	
			
			System.out.println("iterating bag using for-each gives");
			for (IMiniKeyValuePair<IMiniKeyValuePair<Author, CharacterType>, Integer> pair : uniquesBag) {
				System.out.println(pair);
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println(e.toString());
			e.printStackTrace();
			
		}	
		
		try {	
			System.out.println("iterating through keys using uniquesBag.getKeys() gives");
			for (Author author : uniquesBag.getKeys()) {System.out.println(author);}
			System.out.println();
			
			System.out.println("iterating through keys and values " +
					"using uniquesBag.getKeyValuePairs()");
			for (IMiniKeyValuePair<Author,CharacterType> pair :
						uniquesBag.getKeyValuePairs()) {
				System.out.println(pair);
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println(e.toString());
			
		}
		System.out.println();
		try {
			
			Author au = Heyer;
			
			System.out.println("check if uniquesBag contains " + au);
			boolean contains = uniquesBag.contains(au);
			System.out.println("uniquesBag.contains(" + au + ") = " + contains);
			int count = uniquesBag.count(au);
			System.out.println("count for " + au + " is " + count);
			
			System.out.println("get a sequence of all the values and counts for key " 
					+ au);
			System.out.println(" using uniquesBag.getValuesAndCounts("+ au + "):");
			for (IMiniKeyValuePair<CharacterType,Integer> ctcount 
					: uniquesBag.getValuesAndCounts(au)) {System.out.println(ctcount);}
			System.out.println();
			
			System.out.println("try adding duplicate values to the uniqueBag");
			IMiniSequence<CharacterType> aucts = uniquesBag.getValues(au);
			for (CharacterType auct : aucts) {
				InsertCheck isAdded = uniquesBag.add(au, auct);
				System.out.println("result of uniquesBag.add("+ au + ", " + auct + ") is " 
						+ isAdded);
			}
			System.out.println("get a sequence of all the values and counts for key " 
					+ au);
			System.out.println(" using uniquesBag.getValuesAndCounts("+ au + "):");
			for (IMiniKeyValuePair<CharacterType,Integer> ctcount 
					: uniquesBag.getValuesAndCounts(au)) {System.out.println(ctcount);}
			System.out.println();
			
			System.out.println("try adding non-duplicate value to the uniqueBag");
			CharacterType ct = Wizard;
			InsertCheck isAdded = uniquesBag.add(au, ct);
			System.out.println("result of uniquesBag.add("+ au + ", " + ct + ") is " 
					+ isAdded);
			System.out.println("get a sequence of all the values and counts for key " 
					+ au);
			System.out.println(" using uniquesBag.getValuesAndCounts("+ au + "):");
			for (IMiniKeyValuePair<CharacterType,Integer> ctcount 
					: uniquesBag.getValuesAndCounts(au)) {System.out.println(ctcount);}
			System.out.println();
			
			System.out.print("removeOnePair(" + au + ")");
			boolean isRemoved = uniquesBag.removeOnePair(au, ct);
			System.out.println(" = " + isRemoved);
			System.out.println("check if uniquesBag contains " + au);
			contains = uniquesBag.contains(au);
			System.out.println("uniquesBag.contains(" + au + ") = " + contains);
			
			System.out.println("add " + au + " back with original value");
			for (CharacterType auct : aucts) {
				isAdded = uniquesBag.add(au, auct);
				System.out.println("result of uniquesBag.add("+ au + ", " + auct + ") is " 
						+ isAdded);
			}
			
		}
		catch (Exception e) {
			System.out.println(e.toString());
		}
		System.out.println();
		try {	
			Author au = new Author("Asimonov");
			System.out.println("check if uniquesBag contains " + au);
			boolean contains = uniquesBag.contains(au);
			System.out.println("uniquesBag.contains(" + au + ") = " + contains);
			
			CharacterType ct = new CharacterType("Alien");
			InsertCheck isAdded = uniquesBag.add(au, ct);
			System.out.println("result of uniquesBag.add("+ au + ", " + ct + ") is " 
					+ isAdded);
			System.out.println("get a sequence of all the values and counts for key " 
					+ au);
			System.out.println(" using uniquesBag.getValuesAndCounts("+ au + "):");
			for (IMiniKeyValuePair<CharacterType,Integer> ctcount 
					: uniquesBag.getValuesAndCounts(au)) {System.out.println(ctcount);}
			System.out.println();
			System.out.println("replace value for " + au + " with " + ct);
			CharacterType GreenAlien = new CharacterType("Green Alien");
			ct = GreenAlien;
			isAdded = uniquesBag.add(au, ct);
			System.out.println("result of uniquesBag.add("+ au + ", " + ct + ") is " 
					+ isAdded);
			System.out.println("get a sequence of all the values and counts for key " 
					+ au);
			System.out.println(" using uniquesBag.getValuesAndCounts("+ au + "):");
			for (IMiniKeyValuePair<CharacterType,Integer> ctcount 
					: uniquesBag.getValuesAndCounts(au)) {System.out.println(ctcount);}
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println(e.toString());
		}
		System.out.println();
		
		
	}

	
	private static class Author implements Comparable<Author>{
		String name;
		private Author() {}
		public Author(String name) {this.name = name; }
		public String getName() { return name; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			Author that = (Author) o;
			return this.name.equals(that.getName());
		}
		public int hashCode() {
			return name.hashCode();
		}
		public String toString() {return name; }
		public int compareTo(Author that) {
			//int retvalue = 0;
			//if (this.name.length() < that.getName().length()) retvalue = -1;
			//if (this.name.length() < that.getName().length()) retvalue = 1;
			return (this.name.compareTo(that.getName()));
		}
	}
	
	private static class CharacterType {
		String description;
		private CharacterType() {}
		public CharacterType(String description) {this.description = description; }
		public String getDescription() { return description; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			CharacterType that = (CharacterType) o;
			return this.description.equals(that.getDescription());
		}
		public int hashCode() {
			return description.hashCode();
		}
		public String toString() {return description; }
		public int compareTo(CharacterType that) {
			return (this.description.compareTo(that.getDescription()));
		}
	}
	

}
