package testing;

import minicollection.*;

/**
 * @author Jenny Harlow
 * 
 * Test bags of unique singles
 *
 */
public class TestUniqueBagSingles {
	
	public static void main(String[] args) {
		
		IMiniBagSingles<String> uniqueBag = null;
		try {
		
			IFactoryMiniBagSingles<String> factory = new FactoryMiniBagSingles<String>();
			
			IMiniBagSingles<String> bag = factory.makeBagSingles();
			System.out.println("made empty bag");
			
			System.out.println("checking configuration");
			System.out.println("getMembershipMode() = " + bag.getMembershipMode());
			System.out.println();
			
			
			System.out.println("IsEmpty is " + bag.isEmpty());
			System.out.println("Size is " + bag.size());
			
			bag.add("Jenny");
			bag.add("Wal");
			bag.add("Linda");
		
		
			try {
				System.out.println("adding duplicates Wals and Jennys");
				String s = "Wal";
				InsertCheck isAdded = bag.add(s);
				System.out.println("result of add(" + s + ") is " + isAdded);
				 s = "Wal";
				isAdded = bag.add(s);
				System.out.println("result of add(" + s + ") is " + isAdded);
				 s = "Jenny";
				isAdded = bag.add(s);
				System.out.println("result of add(" + s + ") is " + isAdded);
				
				System.out.println("Size is " + bag.size());
				System.out.println("uniqueKeySize is " + bag.uniqueKeySize());
						
				System.out.println("iterating with for each");
				for (IMiniKeyValuePair<String,Integer> pair : bag) {	System.out.print(pair + " ");}
			}
			catch (Exception e) {
				System.out.println("That resulted in an exception:");
				System.out.println(e.toString());
			}	
			System.out.println();
		
		
		IFactoryMiniBagSingles<String> unique_factory = 
			new FactoryMiniBagSinglesUniqueOnly<String>();
		
		uniqueBag = unique_factory.makeBagSingles(bag);
		
		
		System.out.println();
		System.out.println("made unique bag, bag is");
		for(IMiniKeyValuePair<String,Integer> pair : uniqueBag) {System.out.print(pair + "  ");}
		System.out.println();
		System.out.println("checking configuration");
		System.out.println("getMembershipMode() = " + uniqueBag.getMembershipMode());
		System.out.println();
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		
		try {
			
			String s = "Gardinia";
			System.out.println("try adding non-duplicate: check if bag contains " + s);
			System.out.println("bag.contains(" + s + ") = " + uniqueBag.contains(s));
			int count = uniqueBag.count(s);
			System.out.println("count for " + s + " is " + count);
			System.out.println("using bag.add(" + s + "):");
			InsertCheck isAdded = uniqueBag.add(s);
			System.out.println("result of add(" + s + ") is " + isAdded);
			System.out.println("bag.contains(" + s + ") = " + uniqueBag.contains(s));
			System.out.println("count for " + s + " is " + uniqueBag.count(s));
				
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		try {
			String s = uniqueBag.bagIterator().currentKey();
			System.out.println("try adding duplicate: check if bag contains " + s);
			System.out.println("bag.contains(" + s + ") = " + uniqueBag.contains(s));
			int count = uniqueBag.count(s);
			System.out.println("count for " + s + " is " + count);
			System.out.println("using bag.add(" + s + "):");
			InsertCheck isAdded = uniqueBag.add(s);
			System.out.println("result of add(" + s + ") is " + isAdded);
			System.out.println("bag.contains(" + s + ") = " + uniqueBag.contains(s));
			System.out.println("count for " + s + " is " + uniqueBag.count(s));
				
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		
		try {
			
			String s = uniqueBag.bagIterator().currentKey();
			System.out.println("trying remove: check if bag contains " + s);
			System.out.println("bag.contains(" + s + ") = " + uniqueBag.contains(s));
			int count = uniqueBag.count(s);
			System.out.println("count for " + s + " is " + count);
			System.out.println("using bag.remove(" + s + "):");
			uniqueBag.removeOne(s);
			count = uniqueBag.count(s);
			System.out.println("removed one, count for " + s + " is " + uniqueBag.count(s));
			System.out.println("bag.contains(" + s + ") = " + uniqueBag.contains(s));
			
			try {
				System.out.println("try using bag.remove(" + s + ") again:");
				uniqueBag.removeOne(s);
				System.out.println("count for " + s + " is " + uniqueBag.count(s));
			}
			catch (Exception e) {
				System.out.println("That resulted in an exception:");
				System.out.println(e.toString());
			}
			System.out.println();
			
			System.out.println("add " + s + " back in ");
			uniqueBag.add(s);
			System.out.println("bag.contains(" + s + ") = " + uniqueBag.contains(s));
			count = uniqueBag.count(s);
			System.out.println("count for " + s + " is " + uniqueBag.count(s));
			
			
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
			
		try {
			System.out.println("iterating using the BagSinglesIterator");
			IMiniBagSinglesIterator<String> it = uniqueBag.bagIterator();
			while(it.current() != null) {
				System.out.print(it.current() + "  ");
				it.next();
			}
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		
		try {
			System.out.println("iterating using the BagSinglesIterator and currentKey()");
			IMiniBagSinglesIterator<String> it = uniqueBag.bagIterator();
			while(it.current() != null) {
				System.out.print(it.currentKey() + "  ");
				it.next();
			}
			System.out.println();
			
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}	
		System.out.println();
		
		try {
			System.out.println("getting out the sequence of keys and counts");
			IMiniSequence<IMiniKeyValuePair<String, Integer>> keysAndCounts =
				uniqueBag.getKeysAndCounts();
			System.out.println("size is " + keysAndCounts.size() + " and pairs are:");
			for (IMiniKeyValuePair<String, Integer> pair : keysAndCounts) {
				System.out.println(pair);
			}
			System.out.println();
		}
		catch (Exception e) {
			System.out.println("That resulted in an exception:");
			System.out.println(e.toString());
		}
		
	}

}
