package testing;


import java.util.*;

import minicollection.*;

/**
 * @author Jenny Harlow
 * 
 * Test sequences of unique elements with extended index operators
 *
 */
public class TestUniqueSequenceFlexiAdd {
	
	public static void main(String[] args) {

		IFactoryMiniSequence<String> factory = new FactoryMiniSequence<String>();
		
		IMiniSequence<String> seq = factory.makeSequence();
		
		System.out.println("made empty sequence, checking configuration");
		System.out.println("getMembershipMode() = " + seq.getMembershipMode());
		System.out.println("getOrderingMode() = " + seq.getOrderingMode());
		System.out.println("getAccessMode() = " + seq.getAccessMode());
		System.out.println();
		
				
		seq.add("Jenny");
		seq.add("Wal");
		seq.add("Linda");
		seq.add("Gardinia");
		seq.add("Wal");
		seq.add("Wal");
				
		System.out.println("sequence is");
		IMiniSequenceIterator<String> it = seq.begin();
		while(it.current() != null) {
			System.out.print(it.current() + " ");
			it.next();
		}
		System.out.println();
		
		IFactoryMiniSequenceFlexiAdd<String> unique_factory = 
			new FactoryMiniSequenceUniqueOnly<String>();
		
		IMiniSequenceFlexiAdd<String> uniqueSeq = unique_factory.makeSequence(seq);
		
		System.out.println();
		System.out.println("made unique sequence, sequence is");
		for(String ss : uniqueSeq) {System.out.print(ss + "  ");}
		System.out.println();
		System.out.println("made sorted sequence, checking configuration");
		System.out.println("getMembershipMode() = " + seq.getMembershipMode());
		System.out.println("getOrderingMode() = " + seq.getOrderingMode());
		System.out.println("getAccessMode() = " + seq.getAccessMode());
		System.out.println();
		
		String s = "Richard";
		System.out.println("try adding non-duplicate " + s + ":");
		System.out.println("result of add is " + uniqueSeq.add(s));
		System.out.println("sequence now is");
		for(String ss : uniqueSeq) {System.out.print(ss + "  ");}
		System.out.println();
		System.out.println();
		
		s = "Jenny";
		System.out.println("try adding duplicate " + s + ":");
		System.out.println("result of add is " + uniqueSeq.add(s));
		System.out.println("sequence now is");
		for(String ss : uniqueSeq) {System.out.print(ss + "  ");}
		System.out.println();
		System.out.println();
		
		s = "Humperdink";
		int ind = 0;
		System.out.println("try addAtIndex non-duplicate " + s + " at index " + ind +";");
		System.out.println("result of add is " + uniqueSeq.addAtIndex(ind, s));
		System.out.println("sequence now is");
		for(String ss : uniqueSeq) {System.out.print(ss + "  ");}
		System.out.println();
		System.out.println();
		
		
		s = "Humperdink";
		System.out.println("try addAtIndex duplicate " + s + " at index " + ind +";");
		System.out.println("result of add is " + uniqueSeq.addAtIndex(ind, s));
		System.out.println("sequence now is");
		for(String ss : uniqueSeq) {System.out.print(ss + "  ");}
		System.out.println();
		System.out.println();
		
		
		s = "Gardinia";
		int target_index = uniqueSeq.firstIndexOf(s);
		if (target_index >= 0) {
			System.out.println("try addAtIndex duplicate " + s + " at index " + ind +";");
			System.out.println("result of add is " + uniqueSeq.addAtIndex(ind, s));
			System.out.println("sequence now is");
			for(String ss : uniqueSeq) {System.out.print(ss + "  ");}
			System.out.println();
			System.out.println();
		}
		
				
		s = "Marmalade";
		System.out.println("try resetAtIndex non-duplicate " + s + " at index " + ind +";");
		System.out.println("result of resetAtIndex is " + uniqueSeq.resetAtIndex(ind, s));
		System.out.println("sequence now is");
		for(String ss : uniqueSeq) {System.out.print(ss + "  ");}
		System.out.println();
		System.out.println();
		
		s = "Maramalade";
		System.out.println("try resetAtIndex duplicate " + s + " at index " + ind + ":");
		System.out.println("result of resetAtIndex is " + uniqueSeq.resetAtIndex(ind, s));
		System.out.println("sequence now is");
		for(String ss : uniqueSeq) {System.out.print(ss + "  ");}
		System.out.println();
		System.out.println();
		
		s = "Linda";
		target_index = uniqueSeq.firstIndexOf(s);
		if (target_index >= 0) {
			System.out.println("try resetAtIndex duplicate " + s + " at index " + ind +";");
			System.out.println("result of resetAtIndex is " + uniqueSeq.resetAtIndex(ind, s));
			System.out.println("sequence now is");
			for(String ss : uniqueSeq) {System.out.print(ss + "  ");}
			System.out.println();
			System.out.println();
		}
		
		System.out.println("iterating from the end");
		it = uniqueSeq.end();
		while(it.current() != null) {
			System.out.print(it.current() + "  ");
			it.previous();
		}
		System.out.println();
		

		uniqueSeq.sort(new CustomComparator());
		System.out.println("sorted by length, sequence now is");
		for(String ss : uniqueSeq) {System.out.print(ss + "  ");}
		System.out.println();
	}

	private static class CustomComparator
	implements Comparator<String>
	{
		public int compare(String s1, String s2) {
			int retvalue = 0;
			if (s1.length() < s2.length()) retvalue = -1;
			if (s1.length() > s2.length()) retvalue = 1;
			return retvalue;
		}
	}
	

}
