package testing;

import java.util.*;

/**
 * @author Jenny Harlow
 * 
 * Demonstrate how to modify an 'unmodifiable' collection 
 * from the Java Collections Framework
 *
 */
public class TestUnmodifiableList {
	
	
	public static void main(String[] args) {
		
		Set<NaturallyComparable> set = new HashSet<NaturallyComparable>();
		
		set.add(new NaturallyComparable("Jenny"));
		set.add(new NaturallyComparable("Humperdink"));
		set.add(new NaturallyComparable("Gardinia"));
		
		System.out.println("Create a Set of objects which have a String name attribute.  The set is:");
		Iterator<NaturallyComparable> it = set.iterator();
		while (it.hasNext()) { System.out.print(it.next() + " ");}
		System.out.println();
		Set<NaturallyComparable> protectedSet = Collections.unmodifiableSet(set);
		System.out.println("Use Collections.unmodifiableSet(set) to make an unmodifiabe set:" +
		" the unmodifiable set is:");
		Iterator<NaturallyComparable> pit = protectedSet.iterator();
		while (pit.hasNext()) { System.out.print(pit.next() + " ");}
		System.out.println();
		pit = protectedSet.iterator();
		System.out.println("Use the iterator over the unmodifiable set to reset the name to Ubiquitous for all objects:");
		while (pit.hasNext()) {
			pit.next().setName("Ubiquitous");
		}
		pit = protectedSet.iterator();
		System.out.println("The unmodifiable set now is:");

		while (pit.hasNext()) { System.out.print(pit.next() + " ");}
		System.out.println();
		System.out.println("and the original set now is:");
		it = set.iterator();
		while (it.hasNext()) { System.out.print(it.next() + " ");}
		
	}
	
	private static class NaturallyComparable implements Comparable<NaturallyComparable>{
		String name;
		private NaturallyComparable() {}
		public NaturallyComparable(String name) {this.name = name; }
		public String getName() {return name; }
		public void setName(String n) {name = n; }
		public boolean equals(Object o) {
			if (o == this) return true;
			if (o == null) return false;
			if (getClass() != o.getClass()) return false;
			NaturallyComparable that = (NaturallyComparable) o;
			return this.name.equals(that.getName());
		}
		public int hashCode() {
			return name.hashCode();
		}
		public String toString() {return name; }
		public int compareTo(NaturallyComparable that) {
			return (this.name.compareTo(that.getName()));
		}
	}

}
